#include "mainwindow.h"
#include "qapplication.h"
#include "ui_mainwindow.h"
#include <QGraphicsScene>
#include<qdebug.h>
#include<QStyleFactory>
#include<qfiledialog.h>
#include<QSplitter>
#include<QMessageBox>
#include<QStatusBar>
#include"tcad.h"


MainWindow::MainWindow(QString _startopenfile, QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    is_started=0;
    ui->setupUi(this);

    setWindowTitle("ООО 'Матрица', dxf-постпроцессор ");

    QFont newFont("FreeSans", 9, QFont::Light, false);
//    newFont.setLetterSpacing(QFont::PercentageSpacing,95);
    QApplication::setFont(newFont);

    TCAD::CAD=ui->CAD;
    TCAD::MainWin = this;
    TCAD::CAD->toolbar=ui->mainToolBar;

    QVBoxLayout *Layout = new QVBoxLayout(ui->RightFrame);

    TCAD::CAD->ListTool = new TListTool();

    Layout->addWidget(TCAD::CAD->ListTool);
    Layout->addSpacing(20);

    createActions();
    createMenus();

    dbof = new DbOpenForm(this);

    startopenfile = _startopenfile;

    TCAD::CAD->pconf = new TppConfig(this);
    PLA = new TPlatesArrangeForm(this);
    showMaximized();

}

void MainWindow::StartTimerShot()
{
    TimerWhenStart->stop();
}

void MainWindow::showEvent(QShowEvent * event)
{
    QWidget::showEvent(event);

    if(is_started==0)
    {
        is_started=1;
        if( startopenfile != "" )
            TCAD::CAD->OpenDXF( startopenfile );
        else
            TCAD::CAD->OpenDXF( TCAD::CAD->pconf->get_last_opened_file() );
    }
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    delete ui;
}

MainWindow::~MainWindow()
{
}

void MainWindow::createActions()
{
    ReversePolylineAct = new QAction(tr("&Развернуть полилинию"), this);
    ReversePolylineAct->setStatusTip(tr("Разворот полилинии"));
    connect(ReversePolylineAct, SIGNAL(triggered()), TCAD::CAD, SLOT(ReversePolyline()));

    OptimizeAct = new QAction(tr("&Оптимизировать размещение"), this);
    OptimizeAct->setStatusTip(tr("Упорядочение полилиний"));
    connect(OptimizeAct, SIGNAL(triggered()), TCAD::CAD, SLOT(DoOptimize()));

    SetBasePoint = new QAction(tr("Установить базовую точку привязки"), this);
    connect(SetBasePoint, SIGNAL(triggered()), TCAD::CAD, SLOT(SetBasePoint()));
}

void MainWindow::createMenus()
{
    fileMenu = menuBar()->addMenu(tr("&Меню"));
    fileMenu->addAction(tr("&Открыть dxf"), this, SLOT(openFile()),QKeySequence(tr("Ctrl+O")) );
    fileMenu->addAction(tr("&Открыть базу"), this, SLOT(DbOpen()),QKeySequence(tr("Alt+O")) );
    fileMenu->addAction(tr("&Открыть компоновщик"), this, SLOT(PACOpen()),QKeySequence(tr("Alt+P")) );
                                                                               fileMenu->addSeparator();
    fileMenu->addAction(tr("&Рамка"), this, SLOT(ShowRam()),QKeySequence(tr("Alt+E")) );
    fileMenu->addSeparator();
    fileMenu->addAction(tr("&Генерировать код"), this, SLOT(GenerateG()),QKeySequence(tr("Alt+G")) );
    fileMenu->addSeparator();
    fileMenu->addAction(tr("Выход"), this, SLOT(ExitProgram()),QKeySequence(tr("Alt+Q")) );

    confMenu = menuBar()->addMenu(tr("&Настройки"));
    confMenu->addAction(tr("&Параметры"), this, SLOT(CfgOpen()),QKeySequence(tr("Alt+C")) );

    ToolSelectMode = ui->mainToolBar->addAction(tr("Выбор"),this, SLOT(ToSelectMode()));
    ToolSelectMode->setToolTip("Переключить в режим выбора линий");
    ToolSelectMode->setCheckable(true);
    ToolSelectMode->setChecked(1);
    DrawLineMode = ui->mainToolBar->addAction(tr("Отрезок"),this, SLOT(ToDrawLineMode()));
    DrawLineMode->setToolTip("Переключить в режим черчения линий");
    DrawLineMode->setCheckable(true);
    ui->mainToolBar->addSeparator();

    QMenu *tmpmenu = menuBar()->addMenu(tr("Вид"));
    tmpmenu->addAction(tr("Масштаб+"), this, SLOT(ZoomIn()),QKeySequence(Qt::CTRL + Qt::Key_Plus) );
    tmpmenu->addAction(tr("Масштаб-"), this, SLOT(ZoomOut()),QKeySequence(Qt::CTRL + Qt::Key_Minus) );

    CItool = ui->mainToolBar->addAction(tr("CI"),this, SLOT(cmdCI()));
    CItool->setToolTip("Врезка в окружность изнутри");
    CItool->setEnabled(false);
    CCItool = ui->mainToolBar->addAction(tr("CCI"),this, SLOT(cmdCCI()));
    CCItool->setToolTip("Врезка изнутри во все окружности");

    ToRotate = ui->mainToolBar->addAction(tr("Повернуть на 90"),this, SLOT(DoRotate()));
}

void MainWindow::contextMenuEvent(QContextMenuEvent *event)
{
    QMenu menu(this);
//    menu.addAction(openAct);
    menu.addAction(ReversePolylineAct);
    ReversePolylineAct->setEnabled( TCAD::CAD->selectedPolyline!=nullptr );
    menu.addAction(OptimizeAct);

    menu.addAction(SetBasePoint);
    bool flag=0;
    if(TCAD::CAD->selectedPolyline) flag = TCAD::CAD->selectedPolyline->type==1;
    SetBasePoint->setEnabled( flag );
    menu.exec(event->globalPos());
}

void MainWindow::openFile()
{
    QString dir = QDir(TCAD::CAD->pconf->get_last_opened_file()).absolutePath();
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),dir, tr("Images (*.dxf)"));
    if(fileName>"")
      TCAD::CAD->OpenDXF(fileName);
}


void MainWindow::DbOpen()
{
    dbof->ShowForm();
}
void MainWindow::PACOpen()
{
    PLA->showMaximized();
}

void MainWindow::CfgOpen()
{
    TCAD::CAD->pconf->showNormal();
}

void MainWindow::ShowRam()
{
    TCAD::CAD->DisplayRAM(1000,800);
}

void MainWindow::OpenPrimConf()
{
    TCAD::CAD->pconf->setModal(true);
    TCAD::CAD->pconf->show();
}

void MainWindow::ExitProgram()
{
    close();
}

void MainWindow::GenerateG()
{
    TCAD::CAD->PPG->Generate();
}

void MainWindow::SetTitle(QString s)
{
    setWindowTitle("ООО 'Матрица', dxf-постпроцессор для лазера."+s);
}

void MainWindow::SetSelectableLayers()
{
    for(int i=0; i<TCAD::CAD->OrdersList.count(); i++)
    {
      TCAD::CAD->OrdersList[i]->ClearAllSelectionsOnLayers();
    }
    TPolylineContainer *pc ;

    for(int i=0; i<TCAD::CAD->OrdersList.count(); i++)
    {
        pc = TCAD::CAD->OrdersList[i]->Layers;
        while(pc)
        {
          pc->selectable=1;
          pc=pc->next;
        }
    }
}

void MainWindow::SetStatusMessage(QString s)
{
    statusBar()->showMessage(s,5000);
}

void MainWindow::cmdCI()
{
  TCAD::CAD->selectedPolyline->cmdCI();
}

void MainWindow::cmdCCI()
{
  TPolyline::cmdCCI();
}

void MainWindow::SetToolsVisibility(TPolyline *p)
{
    if(!p) return;
    CItool->setEnabled( p->isCircle() || p->isPolygon() );
}

void MainWindow::DoRotate()
{
    TCAD::CAD->Rot();
}

void MainWindow::ToSelectMode()
{
    DrawLineMode->setChecked(false);
    TCAD::CAD->ToolMode=0;
}

void MainWindow::ToDrawLineMode()
{
    ToolSelectMode->setChecked(false);
    TCAD::CAD->ToolMode=1;
}

void MainWindow::ZoomIn()
{
    TCAD::CAD->Zoom(1.2);
}
void MainWindow::ZoomOut()
{
    TCAD::CAD->Zoom(0.8);
}


