#include "tcad.h"
#include<qdebug.h>
#include <QtMath>
#include <QGraphicsItem>
#include <QPainterPath>
#include <QTimer>
#include<qlineedit.h>
#include<QGraphicsSceneHoverEvent>
#include<QButtonGroup>

double LineLen(double x, double y, double x1, double y1);

TCAD* TCAD::CAD=0;
MainWindow* TCAD::MainWin=0;
QString TCAD::main_url="http://192.168.0.2";

TCAD::TCAD(QWidget* w)
   : QGraphicsView(w)
{
    PAF=0;
    is_prepare_scene=0;
//    GC=new TGraphContainer(this);
    TGraphContainer* GC=new TGraphContainer(this);
    OrdersList.append(GC);
    wait_timer_finish=1;
    WaitTimer = new QTimer;
    connect(WaitTimer,SIGNAL(timeout()),this,SLOT(WaitTimerSlot()));
    scalerate=0;
    selectedPolyline=0;
    hoverPolyline=0;
    longtimer=0;
    FlashErrorPolyline=0;
    rubberBand=0;
    NewCreatedLine=0;
    ToolMode=0;
    ListTool=0;
    curr_board_thick=0;
    acc=10; // изменение при масштабировании
    is_preview=0;

    RAMtextx=0;
    RAMtexty=0;
    RAMx=0;
    RAMx1=0;
    RAMy=0;
    RAMy1=0;

    scene = new QGraphicsScene();
    setScene(scene);
    scene->setBackgroundBrush(Qt::black);

    TPolyline::dot_shift=0;
    QTimer *timer = new QTimer(this);
    connect(timer,SIGNAL(timeout()), this ,SLOT(Timer_dot_tick()));
    timer->start(60);
    connect(scene,SIGNAL(selectionChanged()), this ,SLOT(topZSelect()) );

    PPG = new Tpostprocessor;

    newpointedit = new TNewPointEditor(this);
    newpointedit->setParent(this);
    newpointedit->setVisible(false);
//    SMark = new TSchemMark();
}

TCAD::~TCAD()
{
    RAMtextx=0;
    RAMtexty=0;
}



void TCAD::Timer_dot_tick()
{
    longtimer++;
    TPolyline::dot_shift+=1;
    if(TPolyline::dot_shift>23) TPolyline::dot_shift=0;
    QList<QGraphicsItem *> list = scene->selectedItems() ;

    TPolyline *p;
    for(int i=0; i<list.size(); i++)
    {
        p = (TPolyline*)list.at(i);
        p->update();
    }

    if( FlashErrorPolyline )
    {
        flashtimer++;
        if(flashtimer>12)
        {
            FlashErrorPolyline->tryConcatError=0;
            FlashErrorPolyline=0;
        }
        else
        {
            FlashErrorPolyline->tryConcatError = (flashtimer & 2);
            FlashErrorPolyline->update();
        }
    }
}

void TCAD::SetFlashErrorPolyline(TPolyline* pl)
{
    FlashErrorPolyline = pl;
    flashtimer=0;
}

void TCAD::dropAnySelectionFromPolyline(TPolyline*p)
{
    if(selectedPolyline==p) selectedPolyline=0;
    if(hoverPolyline==p) hoverPolyline=0;
    if(FlashErrorPolyline==p) FlashErrorPolyline=0;
}

void TCAD::topZSelect()
{
/*    QGraphicsItem *p;
    QList<QGraphicsItem *> li = scene->items();
    for( int i=0; i < li.size(); i++)
    {
        p = li.at(i);
        p->setZValue(0);
    }

    li = scene->selectedItems();
    for( int i=0; i < li.size(); i++)
    {
        p = li.at(i);
        p->setZValue(10);
    }
*/
}


void TCAD::PrepareGC()
{

 TPolylineContainer* pc;
 for(int i=0; i<OrdersList.count(); i++)
 {
  pc = OrdersList[i]->Layers;

  TGraphContainer *newGC = new TGraphContainer(TCAD::CAD);
  TPolyline *p = pc->Polylines;
  while(p)
  {
      QString s = QString::number(p->color);
      TPolylineContainer *pc1 =  newGC->NewLayer(s, "" );
      TPolyline* p1 = new TPolyline();
      p1->Copy(p);
      pc1->AddPolyline(p1);
      p=p->next;
  }

      TGraphContainer *tmpGC = OrdersList[i];
      newGC->orderinfo = tmpGC->orderinfo;
      OrdersList[i] = newGC;
      delete tmpGC;

 }

}

void TCAD::PrepareGC_fromfile()
{
  TPolylineContainer* pc;
  pc = OrdersList.first()->Layers;
  while(pc)
  {
      if(pc->layer=="DXF-IN")
          break;
      pc=pc->next;
  }

  if(!pc)
  {
      pc = OrdersList.first()->Layers;
      is_prepare_scene=1;
      while(pc)
      {
          pc->MergePolylines();
          pc->MergePolylines();
          pc->MergePolylines();
          pc=pc->next;
      }

      is_prepare_scene=0;

      pc = OrdersList.constFirst()->Layers;
      while(pc)
      {
          if(pc->layer=="DXF-IN") pc->visible=0;
          pc->selectable=pc->visible;
          pc=pc->next;
      }
      return;
  }

  TGraphContainer *newGC = new TGraphContainer(this);
  TPolyline *p = pc->Polylines;
  while(p)
  {
      QString s = QString::number(p->color);
      TPolylineContainer *pc1 =  newGC->NewLayer(s, "" );
      TPolyline* p1 = new TPolyline();
      p1->Copy(p);
      pc1->AddPolyline(p1);
      p=p->next;
  }

  if(this == TCAD::CAD)
  {
      TGraphContainer *tmpGC = OrdersList.first();
      newGC->orderinfo = tmpGC->orderinfo;
      OrdersList.first() = newGC;
      delete tmpGC;
  } else
  {
      delete OrdersList.first();
      OrdersList.first()= newGC;
  }

}

void TCAD::displayLayer(QString layer)
{
/*    TPolylineContainer *pc=GC->Layers;
    while(pc)
    {
        if(pc->layer==layer) break;
        pc=pc->next;
    }
    */
}


void TCAD::SetSelectableLayers()
{
    ClearAllSelectionsOnLayers();
    TPolylineContainer *pc ;
    for(int i=0; i<OrdersList.count(); i++)
    {
      pc = OrdersList[i]->Layers;
        while(pc)
        {
          pc->selectable=1;
          pc=pc->next;
        }
    }
}

void TCAD::ClearAllSelectionsOnLayers()
{
  TPolyline *p;
  for(int i=0; i<OrdersList.count(); i++)
  {
      TPolylineContainer* pc=OrdersList[i]->Layers;
      selectedPolyline=0;
      while(pc)
      {
        p = pc->Polylines;
        while(p)
        {
            p->is_copy=0;
            p = p->next;
        }
        pc=pc->next;
      }
  }
}

TGraphContainer *TCAD::ClearOrders(int need_new)
{
  for(int i=0; i<OrdersList.count(); i++)
  {
      OrdersList[i]->Clear();
      delete OrdersList[i];
  }
  OrdersList.clear();
  if(!need_new) return 0;
  TGraphContainer *gc = new TGraphContainer(this);
  OrdersList.append(gc);
  return gc;
}

void TCAD::PrepareAll(int tozero)
{
  TPolylineContainer* pc;

  for(int i=0; i< OrdersList.count(); i++)
  {
      OrdersList[i]->OptimizeAll();
      OrdersList[i]->SortLayers();
      OrdersList[i]->SetDirectionsPc();
      OrdersList[i]->CalcSheetSize();

      if(tozero) OrdersList[i]->tozero();

      is_prepare_scene=1;

      pc=OrdersList[i]->Layers;
      if(pc)
      {
        pc->MergePolylines();
        pc->MergePolylines();
        pc->MergePolylines();
      }

      is_prepare_scene=0;

      OrdersList[i]->OptimizeAll();
  //   GC->AddBridges();

      pc=OrdersList[i]->Layers;
      while(pc)
      {
          pc->ToScene(scene);
          pc->selectable=pc->visible=1;
          pc=pc->next;
      }
  }

  TCAD::MainWin->SetSelectableLayers();

  QRectF rect = scene->itemsBoundingRect();

  scene->setSceneRect(rect.x()-rect.width(), rect.y()-rect.height(), rect.width()*3, rect.height()*3);

  SetOutRect();

}

int TCAD::OpenDXF(QString filename)
{
  OpenDXF(filename, TOrders("","",filename,"","",""));
  return 0;
}

int TCAD::OpenDXF(QString filename, TOrders info)
{
  TGraphContainer* gc;
  gc = ClearOrders(1);
  if( gc->LoadDxf(filename) )
       return 1;
  gc->orderinfo = info;

  TCAD::CAD->pconf->set_last_opened_file(filename);
  PrepareGC_fromfile();
  PrepareAll(1);
  MainWin->SetTitle(filename);

  return 0;
}

int TCAD::OpenPreview(QString filename, QString layername)
{

  TPolylineContainer* pc;
  if( OrdersList.first()->LoadDxf(filename) )
    return 1;

  pc = OrdersList.first()->Layers;
  while(pc)
  {
    if(pc->layer=="DXF-IN")
        break;
    pc=pc->next;
  }

  if(pc==0) return 0;
  TGraphContainer *newGC =new TGraphContainer(this);
  TPolyline *p = pc->Polylines;
  while(p)
  {
    TPolylineContainer *pc1 =  newGC->NewLayer(layername,"" );
    TPolyline* p1 = new TPolyline();
    p1->Copy(p);
    pc1->AddPolyline(p1);
    p=p->next;
  }
  delete OrdersList.first();
  OrdersList.first()= newGC;

  return 0;
}


void TCAD::SetOutRect()
{
  TGraphContainer* GC = OrdersList.first();
  qreal EX0=GC->EX0;
  qreal EX1=GC->EX1;
  qreal EY0=GC->EY0;
  qreal EY1=GC->EY1;
  for(int i=0; i<OrdersList.count(); i++)
  {
    if(OrdersList[i]->EX0 < EX0) EX0 = OrdersList[i]->EX0;
    if(OrdersList[i]->EY0 < EY0) EY0 = OrdersList[i]->EY0;
    if(OrdersList[i]->EX1 > EX1) EX1 = OrdersList[i]->EX1;
    if(OrdersList[i]->EY1 > EY1) EY1 = OrdersList[i]->EY1;
  }


  centerOn((EX1+EX0)/2, -(EY1+EY0)/2);

  if(CAD == TCAD::CAD)
  {
    TCAD::CAD->ListTool->FillOrderTable();
//    TCAD::CAD->ListTool->LayersModel.Fill();
    TCAD::CAD->ListTool->PLm.pc=0;
    TCAD::CAD->ListTool->PLm.Fill();
  }

  QList<QGraphicsItem *> list = scene->items(Qt::AscendingOrder );
  for(int i=0; i<list.count(); i++)
  {
    if(RAMtextx == list[i])
    {
        delete RAMtextx;
        delete RAMtexty;
        delete RAMx;
        delete RAMx1;
        delete RAMy;
        delete RAMy1;
        break;
    }
  }

  RAMtextx = new QGraphicsTextItem;
  RAMtexty = new QGraphicsTextItem;
  TCAD::CAD->scene->addItem(RAMtextx);
  TCAD::CAD->scene->addItem(RAMtexty);
  RAMx = new QGraphicsLineItem;
  RAMy = new QGraphicsLineItem;
  TCAD::CAD->scene->addItem(RAMx);
  TCAD::CAD->scene->addItem(RAMy);
  RAMx1 = new QGraphicsLineItem;
  RAMy1 = new QGraphicsLineItem;
  TCAD::CAD->scene->addItem(RAMx1);
  TCAD::CAD->scene->addItem(RAMy1);
  //    }

  QString s=QString("%1 мм").arg( EX1-EX0 ) ;
              RAMtextx->setPos((EX1+EX0)/2-30, 8);
  RAMtextx->setPlainText(s);
  QFont font("Times", 16, QFont::Cursive);
  RAMtextx->setFont(font);
  RAMtextx->setDefaultTextColor(0xffffff);

  s=QString("%1 мм").arg( EY1-EY0 );
                                     RAMtexty->setPos(-8, -( EY1+EY0)/2-30);
                           RAMtexty->setRotation(90);
                           RAMtexty->setPlainText(s);
                           RAMtexty->setFont(font);
                           RAMtexty->setDefaultTextColor(0xffffff);
                           QPen pen = QPen(0xffffff);
                           RAMx->setPen(pen);
                           RAMx->setLine(0, 10, 0, 20);
                           RAMy->setPen(pen);
                           RAMy->setLine(-10, 0, -20, 0);
                           RAMx1->setPen(pen);
                           RAMx1->setLine(EX1, 10, EX1, 20);
                           RAMy1->setPen(pen);
                           RAMy1->setLine(-10, -EY1, -20, -EY1);
        SetOutView();
}

void TCAD::SetOutView()
{
                           // -----------
                           QRectF rect1 = QRectF( scene->itemsBoundingRect() );
                           QRectF rect = QRectF(rect1.left()-200, rect1.top()-200, rect1.width()+400, rect1.height()+400);
                           scene->setSceneRect( rect );
                           double w = this->viewport()->width()-100;
                           double h = this->viewport()->height()-100;
                           double wf = this->mapToScene(QPoint(w-1, 0)).x() - this->mapToScene(QPoint(0,0)).x();
                           double hf = this->mapToScene(QPoint(0, h-1)).y() - this->mapToScene(QPoint(0,0)).y();
                           double mx = wf/rect1.width();
                           double my = hf/rect1.height();

                           if(mx<0) mx=1;
                           if(my<0) my=1;

                           if(mx<my)
    this->scale(mx,mx);
                           else
    this->scale(my,my);
                           //   SetSelectableLayers();
                           ensureVisible(rect1);

}

void TCAD::Rot()
{
    TGraphContainer *GC = OrdersList.first();
    GC->Rotate();

    GC->SetDirectionsPc();
    GC->CalcSheetSize();
    GC->tozero();
    GC->OptimizeAll();

    SetOutRect();

    //PrepareAll(0);
    if(ListTool)
    {
        TCAD::CAD->ListTool->FillOrderTable();
//        ListTool->LayersModel.Fill();
        ListTool->PLm.pc=0;
        ListTool->PLm.Fill();
        ListTool->DisplayList();
    }
}


TPolyline* TCAD::FindPolyline(double x, double y, double r)
{
    TPolyline *p;

    if(hoverPolyline)
     {
      if(hoverPolyline->TouchSegment(x,y,r))
       return hoverPolyline;
     }

    for(int i=0; i<OrdersList.count(); i++)
    {
        TPolylineContainer *pc=OrdersList[i]->Layers;
        while(pc)
        {
           if((pc->selectable) )
           {
            p = pc->FindPolyline(x,y,r);
            if(p) return p;
           }
           pc=pc->next;
        }
    }
  return 0;
}


void TCAD::DisplayRAM(float w, float h)
{
  TGraphContainer *GC = OrdersList.first();
    TPolylineContainer* ptr = GC->NewLayer("boardRAM","255");

    TPolyline *p = ptr->Polylines;
    while(p)
    {
        TPolyline*p1=p;
        p=p->next;
        delete p1;
    }
    ptr->Polylines=0;

    p = ptr->NewPolyline();
    p->AddVertex(0,h,0);
    p->AddVertex(0,0,0);
    p->AddVertex(w,0,0);
    p->AddVertex(w,h,0);
    p->AddVertex(0,h,0);
    p->color=255;
    ptr->ToScene(scene);
    ptr->visible=1;
}

void TCAD::SelectPreviewLay(TPolylineContainer *_pc)
{
    for(int i=0; i<OrdersList.count(); i++)
    {
        TPolylineContainer *pc = OrdersList[i]->Layers;
        while(pc)
        {
            pc->preview_selected=0;
            if(pc==_pc) pc->preview_selected=1;
            pc=pc->next;
        }
    }
}

void TCAD::mouseMoveEvent(QMouseEvent * event)
{
    mousePos=event->pos();
    QPointF m = mapToScene(event->pos());
    if( (PAF) && (PAF->SlotMouseMoveEvent(event, QPoint(m.x(), -m.y())) ) )
    {
        return;
    }

    QString s=QString("x=%1 y=%2").arg(m.x()).arg(-m.y()) ;
    MainWin->SetStatusMessage(s);

   if( event->buttons()== Qt::LeftButton )
   {
       QPoint  pos  = event->pos();
       QPointF posf = onmousepressPoint;
       double w = this->viewport()->width();
       double h = this->viewport()->height();
       double wf = this->mapToScene(QPoint(w-1, 0)).x()
                       - this->mapToScene(QPoint(0,0)).x();
       double hf = this->mapToScene(QPoint(0, h-1)).y()
                       - this->mapToScene(QPoint(0,0)).y();
       double lf = posf.x() - pos.x() * wf / w;
       double tf = posf.y() - pos.y() * hf / h;
       this->ensureVisible(lf, tf, wf, hf, 0, 0);
       QPointF newPos = this->mapToScene(pos);

       this->ensureVisible(QRectF(QPointF(lf, tf) - newPos + posf,
                       QSizeF(wf, hf)), 0, 0);
       event->accept();
   }
   else
   {
     if(is_preview)
     {
       TPolylineContainer *needupdate=0;

            TPolylineContainer *pc=0;
       for(int i=0; i<OrdersList.count(); i++)
       {
           pc = OrdersList[i]->Layers;
           while (pc)
           {
               double sx=pc->outer.left();
               double sy= pc->outer.bottom();
               double sx1=pc->outer.right();
               double sy1=pc->outer.top();

                if((m.x() > sx) && (m.x() < sx1) && (-m.y() > sy) && (-m.y() < sy1 ))
                    {
                        needupdate=pc;
                        goto quitlabel;
                    }
            pc=pc->next;
           }
       }
    quitlabel:

       for(int i=0; i<OrdersList.count(); i++)
       {
           pc = OrdersList[i]->Layers;
           while (pc)
           {
            if(pc->preview_hover)
            {
                        if(pc == needupdate)
                        {
                            break;
                        } else
                        {
                            pc->preview_hover=0;
                        }
            }

            pc=pc->next;
           }
       }

       if(needupdate)
       {
            if(pc==0) pc=needupdate;
            needupdate->preview_hover=1;
       }
     }
     else
     {
         TPolyline *p = this->FindPolyline(m.x(),-m.y(),acc);
         if(hoverPolyline !=p)
            {
              TPolyline *p1 = hoverPolyline;
              hoverPolyline=p;
              if(p) p->update();
              if(p1) p1->update();
            }
     }
   }

   QGraphicsView::mouseMoveEvent(event);
}

/*void TCAD::mouseReleaseEvent(QMouseEvent *event)
{
return;
    rubberBand->hide();
    QRect rect = rubberBand->geometry();
    if( (rect.width()>2) || (rect.height()>2))
    {
      QRectF rect_scene = mapToScene(rect).boundingRect();


      qDebug() << rect_scene;
      QList<QGraphicsItem *> list = scene->items(rect_scene,Qt::ContainsItemBoundingRect);
      for (int i = 0; i < list.size(); ++i) {
          list.at(i)->setSelected(1);
      }

    }
}*/

void TCAD::wheelEvent(QWheelEvent * e)
{
    QPointF  pos  = e->position();
                  QPointF posf = this->mapToScene(pos.x(),pos.y());
                  double by;
                  double angle = e->angleDelta().y();
                  if      (angle > 0) { by = 1 + ( angle / 360 * 0.25); }
                  else if (angle < 0) { by = 1 - (-angle / 360 * 0.25); }
                  else                { by = 1; }
                  this->scale(by, by);

                  double w = this->viewport()->width();
                  double h = this->viewport()->height();
                  double wf = this->mapToScene(QPoint(w-1, 0)).x()
                                  - this->mapToScene(QPoint(0,0)).x();
                  double hf = this->mapToScene(QPoint(0, h-1)).y()
                                  - this->mapToScene(QPoint(0,0)).y();
                  double lf = posf.x() - pos.x() * wf / w;
                  double tf = posf.y() - pos.y() * hf / h;
                  this->ensureVisible(lf, tf, wf, hf, 0, 0);
                  QPointF newPos = this->mapToScene(pos.x(),pos.y());
                  this->ensureVisible(QRectF(QPointF(lf, tf) - newPos + posf,
                                  QSizeF(wf, hf)), 0, 0);
                  e->accept();
}

void TCAD::scale(qreal sx, qreal sy)
{
    QGraphicsView::scale(sx,sy);
    QTransform mt = transform();
    acc = 5 / mt.m11();
}

/*void TCAD::mouseDoubleClickEvent(QMouseEvent * )
{}*/

void TCAD::Split(TPolyline* _p, TVertex* sV)
{
    _p->SelectedSegment=0;
    _p->onMouseSegment=0;
    _p->onMouseV=0;
    _p->onMouseSegment=0;

    if(sV->next==0) return;
    if(sV->prev==0) return;
    TVertex *v = _p->firstV;
    while( v && v!=sV )
        v=v->next;
    if(v==sV)
    {
        TPolylineContainer* plc = _p->plcontainer;
        TPolyline *np = plc->NewPolyline();
        np->firstV = new TVertex();
        *np->firstV = *sV;
        np->firstV->next=sV->next;
        sV->next->prev=np->firstV;

        //sV->prev=np->firstV;
        sV->next=0;

        _p->PathRepair();
        np->PathRepair();
        scene->addItem(np);
        np->update();
        _p->update();
    }

}

int TCAD::Concat(TPolyline*p1, TPolyline*p2)
{
    if(!p1->firstV) return 0;
    if(!p2->firstV) return 0;


    TCAD::CAD->hoverPolyline=0;
    TVertex *p1last = p1->firstV->last();
    TVertex *p2last = p2->firstV->last();

    qint64 cx1,cx2,cy1,cy2;


    cx1 = round( 100*p2->firstV->x );
    cx2 = round( 100*p1last->x);
    cy1 = round( 100*p2->firstV->y);
    cy2 = round( 100*p1last->y);
    if( (cx1 == cx2) && (cy1==cy2))
    {

       if(!is_prepare_scene) scene->removeItem(p1);
       if(!is_prepare_scene) scene->removeItem(p2);
        p1->Concat(p2,1);
        if(!is_prepare_scene) scene->addItem(p1);
        goto label1;
    }

    cx1 = round( 100*p1->firstV->x);
    cx2 = round( 100*p2last->x);
    cy1 = round( 100*p1->firstV->y);
    cy2 = round( 100*p2last->y);
    if( (cx1 == cx2) && (cy1==cy2))
//    if( (p1->firstV->x == p2last->x) && (p1->firstV->y == p2last->y))
    {
        if(!is_prepare_scene) scene->removeItem(p1);
        if(!is_prepare_scene) scene->removeItem(p2);
        p1->Concat(p2,0);
        if(!is_prepare_scene) scene->addItem(p1);
        goto label1;
    }

    cx1 = round( 100*p1->firstV->x);
    cx2 = round( 100*p2->firstV->x);
    cy1 = round( 100*p1->firstV->y);
    cy2 = round( 100*p2->firstV->y);



    if( (cx1 == cx2) && (cy1==cy2))
//    if( (p1->firstV->x == p2->firstV->x) && (p1->firstV->y == p2->firstV->y))
    {
        if(!is_prepare_scene) scene->removeItem(p1);
        if(!is_prepare_scene) scene->removeItem(p2);
        p2->Reverse();
        p1->Concat(p2,0);
        if(!is_prepare_scene) scene->addItem(p1);
        goto label1;
    }

    cx1 = round( 100*p1last->x);
    cx2 = round( 100*p2last->x);
    cy1 = round( 100*p1last->y);
    cy2 = round( 100*p2last->y);
    if( (cx1 == cx2) && (cy1==cy2))
//    if( (p1last->x == p2last->x) && (p1last->y == p2last->y))
    {
        if(!is_prepare_scene) scene->removeItem(p1);
        if(!is_prepare_scene) scene->removeItem(p2);
        p2->Reverse();
        p1->Concat(p2,1);
        if(!is_prepare_scene) scene->addItem(p1);
        goto label1;
    }

    return 0;

  label1:

    return 1;
}


void TCAD::mouseReleaseEvent(QMouseEvent *event)
{
    emit MouseReleaseEventSignal(event);
}

void TCAD::mousePressEvent(QMouseEvent * event)
{
   QPoint  pos  = event->pos();
   QPointF posf = this->mapToScene(pos);
   emit MousePressEventSignal(event, QPoint(posf.x(),-posf.y()));
   TPolyline *p;

  if(ToolMode==0)  //-----------------------------ВЫБОР----------------------------------------------
  {

   if( event->buttons() == Qt::LeftButton )
   {
     onmousepressPoint=mapToScene(event->pos());

     if( event->modifiers() == Qt::ControlModifier )
     {
        p=selectedPolyline;
        if(p)
        {
         if(p->onMouseV)
         {
           Split(p,p->onMouseV);
           p->setSelected(true);
         }

         if(selectedPolyline->onMousePoint.next) // если определена красная точка на линии под мышкой
         {
             move_newpointedit(false);
             move_newpointedit(true);
             newpointedit->SetValue(selectedPolyline->LengthofNewVertex(),selectedPolyline->onMouseSegment);
         }
        }
     }

     if( event->modifiers() == Qt::ShiftModifier )
     {
        p=hoverPolyline;
        if( (p && selectedPolyline) && (p!=selectedPolyline)  )
        {
         if( !Concat(selectedPolyline,p) )
           SetFlashErrorPolyline(p);
         else selectedPolyline->setSelected(true);
        }
     }

     if( event->modifiers() == Qt::NoModifier )
     {
       p = hoverPolyline;
       if(p)
       {
          p->setSelected(true);
          p->hoverMoveEvent_nested( posf );  // нажать и заодно выделить
       }
       else
       {
         if(selectedPolyline)  selectedPolyline->setSelected(0);
          selectedPolyline=0;
       }
     }
     DisplayPolyline(selectedPolyline);
   }
  }

  if(ToolMode==1)  //---------------------------------ОТРЕЗОК--------------------------------------------
  {
//      if( event->buttons() == Qt::LeftButton )
//      {
//          if(!NewCreatedLine) NewCreatedLine = GC->Layers->NewPolyline();
//      }
  }

  QGraphicsView::mousePressEvent(event);
}

void TCAD::ClearAllInfo()
{
    if( selectedPolyline )
    {
        selectedPolyline->SelectedSegment=0;
        selectedPolyline->SelectedVertex=0;
    }
}

void TCAD::SelectVertex(TPolyline*p, TVertex *v)
{
    p->SelectedVertex = v;
//    vertexTools->setVisible( (v==0)? 0:1);
//    vertexTools->DisplayInfo(p,v);

    if(v)
    {
//        segTools->setVisible(0);
//        BridgeTools->setVisible(0);
    }
}

void TCAD::SelectSegment(TPolyline*p, TVertex *v)
{
    p->SelectedSegment = v;
//    segTools->setVisible( (v==0)? 0:1);
//    segTools->DisplayInfo(p,v);
//    BridgeTools->setVisible(0);
}


void TCAD::ReversePolyline()
{
    if(selectedPolyline )
       {
        selectedPolyline->Reverse();
        selectedPolyline->setSelected(true);
        selectedPolyline->update();
        ClearAllInfo();
        DisplayPolyline(selectedPolyline);
       }
}

void TCAD::DoOptimize()
{
       for(int i=0; i< OrdersList.count(); i++)
       {
        OrdersList[i]->OptimizeAll();
       }
}

/*void TCAD::SetBasePoint()
{
    double w,h;

    TGraphContainer*GC = OrdersList.first();
    w = GC->EX1-GC->EX0;
    h = GC->EY1-GC->EY0;
    GC->EX0 += TCAD::CAD->selectedPolyline->cx;
    GC->EX1 = GC->EX0+w;
    GC->EY0 += TCAD::CAD->selectedPolyline->cy;
    GC->EY1 = GC->EY0+h;
    GC->tozero();
    SetOutRect();
}
*/

void TCAD::keyPressEvent(QKeyEvent * event)
{
    int key = event->key();
    if(key==70)
    {
        qDebug() << key;
        update();
    }
    if(event->modifiers() != Qt::AltModifier)
    {

    }

     QGraphicsView::keyPressEvent(event);
}


bool TCAD::event(QEvent *event)
{
    QKeyEvent *k = (QKeyEvent *)event;
    if(event->type()==QEvent::KeyPress)
    {
        if ( k->key() == Qt::Key_Control )
        {
        }
    }

    if(event->type()==QEvent::KeyRelease)
    {
        if ( k->key() == Qt::Key_Control )
        {
         if(TCAD::selectedPolyline)
           {
            TCAD::selectedPolyline->onMousePoint.next=0;
            TCAD::selectedPolyline->update();
           }
        }
    }
  QGraphicsView::event(event);
  return true;
}

void TCAD::move_newpointedit(int visible)
{
   if(visible)
   {
    newpointedit->setVisible(true);
    newpointedit->move(mousePos.x()-20,mousePos.y()- 10);
    setFocus();
    newpointedit->len_short->setFocus();
   } else
   {
    newpointedit->setVisible(false);
    CAD->setFocus();
   }
}

void TCAD::focusInEvent (QFocusEvent *)
{
    if(newpointedit->isVisible())
        newpointedit->setVisible(false);
}

void TCAD::DisplayPolyline(TPolyline *p)
{
    if(!p)
    {
        ClearAllInfo();
        ClearAllCopies();
    }
}

void TCAD::ClearAllCopies()
{
   for(int i=0; i< OrdersList.count(); i++)
   {
       TPolylineContainer *pc = OrdersList[i]->Layers;
       while(pc)
       {
           TPolyline *p = pc->Polylines;
           while(p)
           {
               p->is_copy=0;
               p = p->next;
           }

           pc = pc->next;
       }
   }
}

TNewPointEditor::TNewPointEditor(QWidget *parent)
  : QDialog(parent)
{
    this->setWindowFlags(Qt::Dialog | Qt::FramelessWindowHint);

    QHBoxLayout *hb = new QHBoxLayout(this);
    QPushButton *b = new QPushButton("Х");
    hb->addWidget(b);
    b->setMaximumWidth(25);
    connect(b,SIGNAL(clicked(bool)),this,SLOT(Close()));

    QVBoxLayout *Layout = new QVBoxLayout();
    hb->addLayout(Layout);
    len_short = new QLineEdit;
    len_full = new QLineEdit;
    Layout->addWidget(len_short);
    Layout->addWidget(len_full);


    b = new QPushButton("Ok");
    b->setMaximumWidth(25);
    hb->addWidget(b);
    connect(b,SIGNAL(clicked(bool)),this,SLOT(SaveAndClose()));

    validator1 = new QDoubleValidator(0,1000,2);
    validator2 = new QDoubleValidator(0,1000,2);
    len_full->setToolTip("Расстояние до точки от начала полилинии");
    len_short->setToolTip("Расстояние до точки от начала текущего фрагмента");
    len_full->setValidator(validator1);
    len_short->setValidator(validator2);

    connect(len_short,SIGNAL( textChanged(const QString &) ), this ,SLOT(ChangeLenShort()) );
    connect(len_full,SIGNAL( textChanged(const QString &) ), this ,SLOT(ChangeLenFull() ) );
}

void TNewPointEditor::ChangeLenShort()
{
   if(len_short->hasFocus())
   {
    qreal l = len_short->text().toDouble();
    l+= len0;
    len_full->setText( QString().setNum(l,'f',2)  );
   }
}

void TNewPointEditor::ChangeLenFull()
{
   if(len_full->hasFocus())
   {
    qreal l = len_full->text().toDouble();
    l-= len0;
    len_short->setText( QString().setNum(l,'f',2)  );
   }
}

TNewPointEditor::~TNewPointEditor()
{
    delete validator1;
    delete validator2;
}

void TNewPointEditor::keyPressEvent ( QKeyEvent * event )
{
    if(event->key()==Qt::Key_Escape)
    {
        Close();
    }

    if( (event->key()==Qt::Key_Return) || (event->key()==Qt::Key_Enter) )
    {
        SaveAndClose();
    }
    QWidget::keyPressEvent(event);
}


void TNewPointEditor::SetValue(qreal len, TVertex *sv)
{
    TVertex *v = sv->prev;
    len0=0;
    while(v)
    {
        len0+=v->len;
        v=v->prev;
    }

    len_short->setText( QString().setNum(len,'f',2));
    len_full->setText( QString().setNum(len+len0,'f',2));
    startV = sv;
}

qreal TNewPointEditor::GetValue()
{
    return len_short->text().toDouble();
}

void TNewPointEditor::SaveAndClose()
{
    TCAD::CAD->selectedPolyline->ApplyNewVertex( startV, GetValue() );
//    TCAD::CAD->pltool->DisplayInfo(TCAD::CAD->selectedPolyline);
}

void TNewPointEditor::Close()
{
    this->setVisible(false);
}


void TCAD::StartWaitTimer(int msec)
{
    wait_timer_finish=0;
    WaitTimer->start(msec);
}

void TCAD::StopWaitTimer()
{
    wait_timer_finish=1;
    WaitTimer->stop();
}

int TCAD::IsWaitTimerFinish()
{
   return wait_timer_finish;
}

void TCAD::WaitTimerSlot()
{
    wait_timer_finish=1;
}

void TCAD::paintEvent( QPaintEvent * event)
{
    QGraphicsView::paintEvent(event);

        QPainter painter;
        painter.begin(viewport());
//        QPen pen = QPen(0xffeeff);
//        painter.setPen(pen);
//        QBrush brushSel(0x8888ee);

//        painter.setPen(pen);
//        painter.setBrush(brushSel);
//        painter.drawEllipse( 100, 100, 6, 6 );
        painter.end();
 }

void TCAD::Zoom(float m )
{
    double by;
    this->scale(m, m);
}
