#ifndef TCAD_H
#define TCAD_H

#include <QWidget>
#include <QPainter>
#include <QResizeEvent>
#include "tgraphcontainer.h"
#include "tpolyline.h"
#include <QGraphicsScene>
#include <QGraphicsView>
#include <QPointF>
#include<QToolBar>
#include<QRubberBand>

#include<qlabel.h>
#include<QLineEdit>
#include<QVBoxLayout>
#include<qdialog.h>

#include<QTimer>
#include"Tpostprocessor.h"
#include"tppconfig.h"
#include"mainwindow.h"
#include"tpltools.h"
#include"tschemmark.h"

class TPlatesArrangeForm;

class TNewPointEditor : public QDialog
{
    Q_OBJECT
  public:
    QLineEdit *len_short;
    QLineEdit *len_full;
    TVertex *startV;
    QDoubleValidator *validator1;
    QDoubleValidator *validator2;
    qreal len0;

    TNewPointEditor(QWidget *parent);
    ~TNewPointEditor();

    void SetValue(qreal len, TVertex*);
    qreal GetValue();

 protected:
    void keyPressEvent ( QKeyEvent * event );

 public slots:
    void ChangeLenShort();
    void ChangeLenFull();
    void SaveAndClose();
    void Close();
};


class MainWindow;
class TListTool;
class TTableWidget;


// Окно с чертежом на форме mainwindow.ui
class TCAD : public QGraphicsView
{
     Q_OBJECT
private:
    int wait_timer_finish;
    QTimer * WaitTimer;

public:
    QList<TGraphContainer*> OrdersList;
    static TCAD* CAD;
    static MainWindow* MainWin;
    static QString main_url;
    QToolBar *toolbar;
    TNewPointEditor *newpointedit;
    qreal acc; // точность поиска мышкой

    int is_prepare_scene; //
    int XP_prohibit;
    int XN_prohibit;
    int YP_prohibit;
    int YN_prohibit;

    double MatrixShiftX0;
    double MatrixShiftY0;

    TListTool * ListTool;
    Tpostprocessor *PPG;
    TppConfig* pconf;
    int curr_board_thick;

    int is_preview; //

    int ToolMode;  // 0 - выбор, 1 - чертить отрезок
    TPolyline *NewCreatedLine;


    QRubberBand *rubberBand;
    QPoint origin;

    void StartWaitTimer(int msec);
    void StopWaitTimer();
    int IsWaitTimerFinish();

     /// TMP
    QPoint mousePos;

    QGraphicsScene *scene;

//    TSchemMark *SMark;
    QGraphicsTextItem *RAMtextx, *RAMtexty;
    QGraphicsLineItem *RAMx, *RAMy, *RAMx1, *RAMy1;

    double scalerate;
    QPointF onmousepressPoint;
    TPolyline * selectedPolyline;
    TPolyline * hoverPolyline;
    TPolyline * FlashErrorPolyline;
    qlonglong longtimer;
    qlonglong flashtimer;

    explicit TCAD(QWidget*);
    ~TCAD();
    int OpenDXF(QString filename);
    int OpenDXF(QString filename, TOrders info);
    int OpenPreview(QString filename, QString layername);
    void Rot();
    void SetOutRect();
    void SetOutView();
    TGraphContainer *ClearOrders(int need_new);

//    TGraphContainer *GC;
    void SetFlashErrorPolyline(TPolyline*);
    void dropAnySelectionFromPolyline(TPolyline*);
    void move_newpointedit(int visible);
    void scale(qreal sx, qreal sy);
    void DisplayPolyline(TPolyline *p);

    void DisplayRAM(float w, float h);
    void SelectPreviewLay(TPolylineContainer *_pc);

    void PrepareGC();
    void PrepareGC_fromfile();
    void PrepareAll(int tozero);
    void displayLayer(QString layer);
    TPolyline * FindPolyline(double x, double y, double r);

    void SetSelectableLayers();
    void ClearAllSelectionsOnLayers();

    void Split(TPolyline*, TVertex*);
    int Concat(TPolyline*, TPolyline*);

    void ClearAllCopies();

    void SelectVertex(TPolyline*p, TVertex *v);
    void SelectSegment(TPolyline*p, TVertex *v);
//    void SelectChainToolVertex(TPolyline *p, TChainVertex *cv);
    void ClearAllInfo();
    void Zoom(float m );

    TPlatesArrangeForm* PAF;

protected:
    void mouseMoveEvent(QMouseEvent * event);
    void mousePressEvent(QMouseEvent * event);
    void mouseReleaseEvent(QMouseEvent *event);
    void wheelEvent(QWheelEvent * event);
//    void mouseDoubleClickEvent(QMouseEvent * event);
//    void mouseReleaseEvent(QMouseEvent *event);
    void keyPressEvent(QKeyEvent * event);
    bool event(QEvent *event);
    void focusInEvent (QFocusEvent *);
    void paintEvent(QPaintEvent * event);

signals:
//    void MouseMoveEventSignal(QMouseEvent * event, QPointF posf);
    void MousePressEventSignal(QMouseEvent * event, QPointF posf);
    void MouseReleaseEventSignal(QMouseEvent * event);

public slots:
    void Timer_dot_tick();
    void topZSelect();
    void ReversePolyline();
    void WaitTimerSlot();
    void DoOptimize();
//    void SetBasePoint();
};



#endif // TCAD_H
