/*
 * TPolyline - незамкнутая полилиния. К ней приводятся все примитивы из DXF
 *
 * TPolylinecontainer - список полилиний в одном слое layer
 *
 * TGraphContainer содержит набор слоёв
*/

#include "tgraphcontainer.h"
#include <QFile>
#include <QDebug>
#include<tcad.h>
#include<QPainterPath>


ZeroCoordMarker::ZeroCoordMarker()
{
    xline = new TPolyline;
    xline->AddVertex(0,0,0);
    xline->AddVertex(50,0,0);
    xlinearrow1=new TPolyline;
    xlinearrow1->AddVertex(50,0,0);
    xlinearrow1->AddVertex(45,4,0);
    xlinearrow2=new TPolyline;
    xlinearrow2->AddVertex(50,0,0);
    xlinearrow2->AddVertex(45,-4,0);

    xline->color = 9;
    xlinearrow1->color=9;
    xlinearrow2->color=9;

    yline = new TPolyline;
    yline->AddVertex(0,0,0);
    yline->AddVertex(0,50,0);
    ylinearrow1=new TPolyline;
    ylinearrow1->AddVertex(0,50,0);
    ylinearrow1->AddVertex(4,45,0);
    ylinearrow2=new TPolyline;
    ylinearrow2->AddVertex(0,50,0);
    ylinearrow2->AddVertex(-4,45,0);

    yline->color = 9;
    ylinearrow1->color=9;
    ylinearrow2->color=9;

    //    TCAD::CAD->scene->addItem( xline );
    //    TCAD::CAD->scene->addItem( xlinearrow1 );
    //    TCAD::CAD->scene->addItem( xlinearrow2 );
    //    TCAD::CAD->scene->addItem( yline );
    //    TCAD::CAD->scene->addItem( ylinearrow1 );
    //    TCAD::CAD->scene->addItem( ylinearrow2 );
}

ZeroCoordMarker::~ZeroCoordMarker()
{

}


TGraphContainer::TGraphContainer(TCAD * _cad)
{
    Layers = 0;
    CAD = _cad;
}

TGraphContainer::~TGraphContainer()
{
    Clear();
    Layers = nullptr;
}

void TGraphContainer::Clear()
{
    Layers=0;
    CAD->selectedPolyline=0;
    CAD->hoverPolyline=0;
    CAD->scene->clear();
}


void TGraphContainer::tozero()
{
    TPolylineContainer *pc = Layers;
    while(pc)
    {
        TPolyline *p = pc->Polylines;
        while(p)
        {
            if(p->type==1)
            {
                p->cx = p->cx- EX0;
                p->cy = p->cy- EY0;
            }
            TVertex *v = p->firstV;
            while(v)
            {
                v->x = v->x - EX0;
                v->y = v->y - EY0;

                v=v->next;
            }
            p->CalcLength();
            p->PathRepair();

            p=p->next;
        }
        pc=pc->next;
    }

//    PlaceOutsize();
    EX1=EX1-EX0;
    EX0=0;
    EY1=EY1-EY0;
    EY0=0;

}

void TGraphContainer::Rotate()
{
    TPolylineContainer *pc = Layers;
    while(pc)
    {
        TPolyline *p = pc->Polylines;
        while(p)
        {
            double tmp1x = p->cx;
            p->cx = - p->cy;
            p->cy = tmp1x;

            TVertex *v = p->firstV;
            while(v)
            {
                double tmpx = v->x;
                v->x = - v->y;
                v->y = tmpx;

                v=v->next;
            }
            p->CalcLength();
            p->PathRepair();

            p=p->next;
        }
        pc=pc->next;
    }
}


void TGraphContainer::TGraphContainer::OptimizeAll()
{
    double x=0;
    double y=0;
    TPolylineContainer *pc=Layers;
    while(pc)
    {
        OptimizeLay(pc, &x, &y);
        x=0; y=0;
        OptimizeLay(pc, &x, &y);
        x=0; y=0;
        OptimizeLay(pc, &x, &y);
        pc=pc->next;
    }
}


void TGraphContainer::OptimizeLay(TPolylineContainer *pc, double *initial_x, double *initial_y)
{
    TPolyline *p = pc->Polylines;
    while(p)
    {
        QList<QGraphicsItem *> list = CAD->scene->items( *p->path , Qt::ContainsItemShape, Qt::AscendingOrder );
        for (int i = 0; i < list.size(); ++i) {
            QGraphicsItem *gi = list.at(i);
            TPolyline *plt =  static_cast<TPolyline*>(gi);
            plt->PartContour = p;
        }
        p=p->next;
    }

    TPolyline *start_p = pc->Polylines;
    while(start_p)
    {
      if(start_p->PartContour==0) break;
      start_p=start_p->next;
    }

    TPolyline *pfound = pc->FindNearestVertex(pc->Polylines, *initial_x, *initial_y, 0, 0);
    if(!pfound)
        return;
    pc->MoveP(pfound,pc->Polylines);
    p=pfound->next;
    pc->Polylines = pfound;
    while(p)
    {
        pfound = pc->FindNearestVertex(p, pfound->lastV()->x, pfound->lastV()->y, 0, pfound);
        if(pfound)
        {
            pc->MoveP(pfound, p);
            p = pfound->next;
        } else break;
    }

    p = pc->Polylines;
    while(p && (p->PartContour==nullptr))
    {
        pfound = p;
        double x = pfound->lastV()->x;
        double y = pfound->lastV()->y;
        while(pfound)
        {
            pfound = pc->FindNearestVertex(p, x, y, p, pfound);
            if(pfound)
            {
                x = pfound->lastV()->x;
                y = pfound->lastV()->y;
                pc->MoveP(pfound, p);
                if(!pfound->prev)
                      pc->Polylines=pfound;
                pfound=pfound->next;
            }
        }

        p=p->next;
    }

    *initial_x = pc->Polylines->last()->lastV()->x;
    *initial_y = pc->Polylines->last()->lastV()->y;
}

void TGraphContainer::SetDirectionsPc()
{
    TPolylineContainer *pc = Layers;
    while(pc)
    {
//      int col = pc->layer.toInt();
//      if((col>10 ) && (col<90))
      {
        TPolyline *p = pc->Polylines;
        while(p)
        {
            double x0 = p->firstV->x;
            double x1 = p->lastV()->x;
            double y0 = p->firstV->y;
            double y1 = p->lastV()->y;
            double a = VectorToRadial(x0,y0,x1,y1);

            a=a*180/M_PI;

            if( TCAD::CAD->XP_prohibit)
            {
                      if( ((a<45) && ( a>-45)) || ((a<425) && (a>335)))  p->Reverse();
            }
            if( TCAD::CAD->XN_prohibit)
            {
                      if((a>135) && (a<225))  p->Reverse();
            }
            if( TCAD::CAD->YP_prohibit)
            {
                      if((a>45) && (a<135))  p->Reverse();
            }
            if( TCAD::CAD->YN_prohibit)
            {
                      if((a>225) && (a<335))  p->Reverse();
            }
            p=p->next;
        }
       }
       pc=pc->next;
    }

}

TPolylineContainer *TGraphContainer::last()
{
    TPolylineContainer* ptr = Layers;
    if(!ptr) return 0;
    if(!ptr->next) return ptr;
    while(ptr->next) ptr=ptr->next;
    return ptr;
}

TPolylineContainer* TGraphContainer::NewLayer(QString layer, QString _color)
{
    TPolylineContainer* ptr;
    if(!Layers)
     {
         Layers = new TPolylineContainer(layer, _color, this);
         ptr=Layers;
     }
     else
     {
         ptr = Layers;
         while(ptr)
         {
             if(ptr->layer==layer) break;
             ptr=ptr->next;
         }
         if(ptr) return ptr;

         ptr=last();
         ptr->next = new TPolylineContainer(layer, _color, this);
         ptr->next->prev=ptr;
         ptr = ptr->next;
     }
     return ptr;
}
void TGraphContainer::Remove(TPolylineContainer*pc)
{
     TPolylineContainer *tpc = Layers;
     while(tpc)
     {
         if(tpc == pc)
         {
             if(tpc->prev) tpc->prev->next = tpc->next;
             if(tpc->next) tpc->next->prev = tpc->prev;
             delete tpc;
             break;
         }
         tpc = tpc->next;
     }
}

TPolylineContainer *TGraphContainer::NewLayerAtBegin(QString layer, QString _color)
{
    TPolylineContainer* ptr;
    if(!Layers)
     {
         Layers = new TPolylineContainer(layer, _color, this);
         ptr=Layers;
     }
     else
     {
         ptr = Layers;
         while(ptr)
         {
             if(ptr->layer==layer) break;
             ptr=ptr->next;
         }
         if(ptr) return ptr;

         ptr=Layers;
         Layers = new TPolylineContainer(layer, _color, this);
         Layers->next = ptr;
         ptr->prev=ptr;
         ptr = Layers;
     }
     return ptr;
}

TPolyline* TGraphContainer::NewPolyline(QString layer, QString _color)
{
   TPolylineContainer* ptr = NewLayer(layer, _color);
   return ptr->NewPolyline();
}

void TGraphContainer::ChangeLayer(TPolyline* _pl, QString _layer, QString _color)
{
    if(_pl->plcontainer->layer==_layer) return;

    if(_pl->prev) _pl->prev->next=_pl->next;
    if(_pl->next) _pl->next->prev=_pl->prev;
    if((_pl->next==0) && (_pl->prev==0))
          _pl->plcontainer->Polylines=0;
    _pl->next=0;
    _pl->prev=0;
    _pl->plcontainer=0;

    TPolylineContainer *pc=SelectLayer(_layer);
    if(!pc) pc=NewLayer(_layer, _color);
    pc->AddPolyline(_pl);
}


void TGraphContainer::MovePC(TPolylineContainer* src, TPolylineContainer* dest)
{
    if(src==dest) return;
    TPolylineContainer *tmp1 = src->next;
    TPolylineContainer *tmp2 = src->prev;

    if(dest->prev) dest->prev->next = src;
    src->prev = dest->prev;
    dest->prev = src;
    src->next=dest;

    if(tmp1) tmp1->prev = tmp2;
    if(tmp2) tmp2->next = tmp1;
}

void TGraphContainer::AddBridges()
{
    double bridge_len;

    TPolylineContainer* pc;
    pc = Layers;
    while(pc)
    {
          if((pc->layer=="13") || (pc->layer=="23") || (pc->layer=="33") )
          {
            TPolyline *p = pc->Polylines;
            while(p)
            {
                TPolyline *p1 = pc->Polylines;
                while(p1)
                {
                        bridge_len = LineLen( p->lastV()->x, p->lastV()->y, p1->firstV->x, p1->firstV->y  );
                        if( (bridge_len < 6.6) and (bridge_len>2.9) )
                        {

                                  double a1 = p->VectorAngle(1);
                                  double a2 = VectorToRadial(p->lastV()->x, p->lastV()->y, p1->firstV->x, p1->firstV->y);
                                  double da = qAbs(a1-a2);
                                  if( da < 0.1 )
                                  {
                                    TPolyline *pnew = NewPolyline( pc->layer+"_1", "" );
                                    pnew->AddVertex(p->lastV()->x, p->lastV()->y,0);
                                    pnew->AddVertex(p1->firstV->x, p1->firstV->y,0);
                                  }
                        }

                        bridge_len = LineLen( p->firstV->x, p->firstV->y, p1->firstV->x, p1->firstV->y  );
                        if( (bridge_len < 6.6) and (bridge_len>2.9) )
                        {

                                  double a1 = p->VectorAngle(0);
                                  double a2 = VectorToRadial(p1->firstV->x, p1->firstV->y, p->firstV->x, p->firstV->y);
                                  double da = qAbs(a1-a2);
                                  if( da < 0.1 )
                                  {
                                    TPolyline *pnew = NewPolyline( pc->layer+"_1", "" );
                                    pnew->AddVertex(p->firstV->x, p->firstV->y,0);
                                    pnew->AddVertex(p1->firstV->x, p1->firstV->y,0);
                                  }
                        }

                        p1=p1->next;

                }
                p=p->next;
             }
          }
          pc=pc->next;
    }
}

void TGraphContainer::SortLayers()
{
    TPolylineContainer *pc = Layers;

    int col=0;
    TPolylineContainer *tmp=0;

    do
    {
          col=255;
          tmp=0;
          while(pc)
          {
             if( pc->layer.toInt()<col )
             {
            col = pc->layer.toInt();
            tmp=pc;
             }
             pc=pc->next;
          }

          if(tmp)
          {
             MovePC(tmp, Layers);
             pc = tmp->next;
             if(pc) Layers = pc;
          }
    } while (tmp);

    pc = Layers;
    while(pc)
    {
          Layers = pc;
          pc=pc->prev;
    }
}

int TGraphContainer::LoadDxf(QString filename)
{
//    Clear();

    QFile file(filename);
    if(!file.open(QIODevice::ReadOnly))
    {
        qDebug() << "Could not open " << filename;
        return 1;
    }

    QTextStream in(&file);
    QString line, line1;
    TPolyline *pl=0;
//    long
            double x,y,x1,y1,b,r,a1,a2;
    int closed=0;

    while(!in.atEnd())
    {
      line = in.readLine().trimmed();


      if(line=="$EXTMIN")
      {
          while( (line !=" 10") && !in.atEnd() )    line = in.readLine();
           line = in.readLine().trimmed();
           EXTMINX = line.toDouble();
           while( (line !=" 20") && !in.atEnd() )    line = in.readLine();
            line = in.readLine().trimmed();
           EXTMINY = line.toDouble();
      }
      if(line=="$EXTMAX")
      {
          while( (line !=" 10") && !in.atEnd() )    line = in.readLine();
            line = in.readLine().trimmed();
            EXTMAXX =  line.toDouble();
           while( (line !=" 20") && !in.atEnd() )    line = in.readLine();
            line = in.readLine().trimmed();
            EXTMAXY = line.toDouble();

            EXT_W=EXTMAXX-EXTMINX;
            EXT_H=EXTMAXY-EXTMINY;
         qDebug()<< "EXTMINX"<< EXTMINX << "EXTMINY"<<EXTMINX<<  "EXT_W: "<<EXT_W<<  "EXT_H: "<<EXT_H;
      }
      if(line=="ENTITIES") break;
    }

    while(!in.atEnd())
    {
      line = in.readLine();//.trimmed();

      if(line == "ENDSEC")
          break;

      if(line=="CIRCLE")
      {
       x=y=r=0;
       QString color="";
       while((line != "  0") && !in.atEnd())
       {
        line = in.readLine();
        if(line =="  8")
        {
         line = in.readLine().trimmed();
         pl = NewPolyline(line, color);
        }

        if(line ==" 10")
        {
         line = in.readLine().trimmed();
         x = line.toDouble();
        }
        if(line ==" 20")
        {
         line = in.readLine().trimmed();
         y = line.toDouble();
        }
        if(line ==" 40")
        {
         line = in.readLine().trimmed();
         r = line.toDouble();
        }

        if(line==" 62") // color
        {
            line = in.readLine().trimmed();
            color = line;
            pl->SetLayerColor(color);
        }

       }
          TVertex *v;
//         qDebug() << tmp;
//         qDebug() << y;
//         qDebug() << r;
          v = pl->AddVertex(x+r,y,1);
          v = pl->AddVertex(x-r,y,1);
          v = pl->AddVertex(x+r,y,0);
          pl->color=color.toInt();

          if(r<2.5)
          {
            pl->type=1;
            pl->cx=x;
            pl->cy=y;
            pl->cr=r;
            pl->PathRepair();
         }


      }

      if(line=="ARC")
      {
       x=y=r=a1=a2=0;
       QString color;
       while((line != "  0") && !in.atEnd())
       {
        line = in.readLine();

        if(line =="  8")
        {
         line = in.readLine().trimmed();
         pl = NewPolyline(line, color);
        }

        if(line ==" 10")
        {
         line = in.readLine().trimmed();
         x = line.toDouble();
        }
        if(line ==" 20")
        {
         line = in.readLine().trimmed();
         y = line.toDouble();
        }
        if(line ==" 40")
        {
         line = in.readLine().trimmed();
         r = line.toDouble();
        }
        if(line ==" 50")
        {
         line = in.readLine().trimmed();
          a1 = line.toDouble();
        }
        if(line ==" 51")
        {
         line = in.readLine().trimmed();
          a2 = line.toDouble();
        }

        if(line==" 62") // color
        {
            line = in.readLine().trimmed();
            color = line;
            pl->SetLayerColor(color);
        }
       }
         pl->AddArc(x,y,r,a1,a2);
         pl->color=color.toInt();
      }

      if(line=="LINE")
      {
       x=y=x1=y1=0;
       QString color;
       while((line != "  0") && !in.atEnd())
       {
         line = in.readLine();
         if(line=="  8") // layer
         {
          line = in.readLine().trimmed();
          pl = NewPolyline(line, color);
         }

         if(line==" 10")
         {
          line = in.readLine().trimmed();
          x = line.toDouble();
         }
         if(line ==" 20")
         {
          line = in.readLine().trimmed();
          y = line.toDouble();
         }


         if(line ==" 11")
         {
          line = in.readLine().trimmed();
          x1 = line.toDouble();
         }
         if(line ==" 21")
         {
          line = in.readLine().trimmed();
          y1 = line.toDouble();
         }

         if(line==" 62") // color
         {
             line = in.readLine().trimmed();
             color=line;
             pl->SetLayerColor(color);
         }

       }
       if(pl)
       {
        pl->AddVertex(x,y,0);
        pl->AddVertex(x1,y1,0);
        pl->color=color.toInt();
       }
      }

      if(line=="POLYLINE")
      {
        closed=0;
        QString color;
        while((line != "SEQEND") && !in.atEnd())
         {
            line = in.readLine();
            if(line=="  8") // layer
            {
                line = in.readLine().trimmed();
                pl = NewPolyline(line, color);
            }

            if(line==" 62") // color
            {
                line = in.readLine().trimmed();
                pl->color=line.toInt();
                pl->SetLayerColor(color);
            }


            if(line=="VERTEX")
            {   x=y=b=0;


                line = in.readLine().trimmed();
                while((line!=" 10")&&(line!="SEQEND")) line= in.readLine();
                if(line==" 10")
                {
                    line = in.readLine().trimmed();
                    x=line.toDouble();
                    line = in.readLine(); //20
                    line = in.readLine().trimmed();
                    y=line.toDouble();
                    while((line!=" 42")&&(line!="  0")) line= in.readLine();
                    if(line==" 42")
                    {
                        line = in.readLine().trimmed();
                        b=line.toDouble();
                    }
                    pl->AddVertex(x,y,b);
                }
            }
         }
      }

      if(line=="LWPOLYLINE")
      {
        int vcnt=0;
        closed=0;
        QString color;
          while((line != "AcDbEntity") && !in.atEnd())
                         line = in.readLine();
          line = in.readLine();
          if(line=="  8") // layer
          {
              line = in.readLine();//.trimmed();
              pl = NewPolyline(line, color);
          }

          while((line != "AcDbPolyline") && !in.atEnd())
          {
              line = in.readLine();
              if(line==" 62") // color
              {
                  line = in.readLine().trimmed();
                  color = line;
                  pl->color=color.toInt();
                  pl->SetLayerColor(color);
              }
          }
          line = in.readLine();//.trimmed();
          if(line==" 90")
          {
              line = in.readLine().trimmed();
              vcnt=line.toInt();
          }
          while((line != " 10") && !in.atEnd())
          {
              line = in.readLine();//.trimmed();
              if(line==" 70") //
              {
                  line = in.readLine().trimmed();
                  closed=line.toInt();
              }
          }
          for(int i=0; i<vcnt; i++)
          { x=y=b=0;
            line = in.readLine();//.trimmed();
            x=line.toDouble();

            while( (line!=" 10") )
            {
              line = in.readLine();
              if(line==" 20")
                {
                  line1 = in.readLine();
                  y=line1.toDouble();
                }

              if(line==" 42")
                {
                  line1 = in.readLine();
                  b=line1.toDouble();
                }

              if(line=="  0") break;

            }
            pl->AddVertex(x,y,b);
                //qDebug()<<x<<y<<b;
          }

         if(closed) pl->close();
      }
    }

//    CreateOffsetLinesLayer();
    return 0;
}

void TGraphContainer::CreateOffsetLinesLayer()
{
    TPolylineContainer* pc = SelectLayer("WideRules");
    if(pc)
    {
        if(pc->prev) pc->prev->next = pc->next;
        if(pc->next) pc->next->prev = pc->prev;
        delete pc;
    }

    pc = NewLayer("WideRules", "255");

    TPolylineContainer* pcS = SelectLayer("R");
    TPolyline* p = 0;
    if(pcS) p = pcS->Polylines;
    while(p)
    {
        pc->CreateOffsetLines(p);
        p = p->next;
    }

    p=0;
    pcS = SelectLayer("PERF");
    if( pcS ) p = pcS->Polylines;
    while(p)
    {
        pc->CreateOffsetLines(p);
        p = p->next;
    }

    p=0;
    pcS = SelectLayer("TECH");
    if(pcS) p = pcS->Polylines;
    while(p)
    {
        pc->CreateOffsetLines(p);
        p = p->next;
    }
    pcS = SelectLayer("B");
    if(pcS) p = pcS->Polylines;
    while(p)
    {
        pc->CreateOffsetLines(p);
        p = p->next;
    }

//  displayLayer("WideRules");

}


TPolylineContainer* TGraphContainer::SelectLayer(QString layer)
{
    TPolylineContainer* pc=Layers;
    while(pc)
    {
        if(pc->layer==layer) return pc;
        pc=pc->next;
    }
    return 0;
}

void TGraphContainer::SetNonselecableAll()
{
    TPolylineContainer* pc=Layers;
    while(pc)
    {
        pc->selectable=0;
        pc=pc->next;
    }
}



TPolylineContainer *TGraphContainer::FindByNo(int no)
{
  TPolylineContainer * pc = Layers;
  int i=0;
  while(pc)
   {
      if(i==no) return pc;
      i++;
      pc=pc->next;
   }
  return 0;
}


TPolyline* TGraphContainer::FirstP()
{
    queuePC = Layers;
    queueP = Layers->Polylines;
    return Layers->Polylines;
}

TPolyline* TGraphContainer::NextP()
{
    queueP = queueP->next;
    if(!queueP)
    {
        queuePC = queuePC->next;
        if(queuePC) queueP = queuePC->Polylines;
        else return 0;
    }
    return queueP;
}


void TGraphContainer::CalcSheetSize()
{
    TPolylineContainer *pc = Layers;
    if(!pc) return;
    TPolyline *p = pc->Polylines;
    p->CalcMinMax();
    EX0 = p->x_min;
    EY0 = p->y_min;
    EX1 = p->x_max;
    EY1 = p->y_max;
    p=p->next;

    while(pc)
    {
      p=pc->Polylines;
      if(p)
      {
       while(p)
       {
         p->CalcMinMax();
         if(p->x_min < EX0) EX0=p->x_min;
         if(p->y_min < EY0) EY0=p->y_min;
         if(p->x_max > EX1) EX1=p->x_max;
         if(p->y_max > EY1) EY1=p->y_max;
         p=p->next;
       }
      }
      pc=pc->next;
    }
}



void TGraphContainer::ClearAllSelectionsOnLayers()
{
    TPolyline *p;
    TPolylineContainer* pc=Layers;
    TCAD::CAD->selectedPolyline=0;
    while(pc)
    {
      p = pc->Polylines;
      while(p)
      {
       p->is_copy=0;
       p = p->next;
      }
      pc=pc->next;
    }


    pc=Layers;

}


