#include "tpltools.h"
#include<QPalette>
#include"tcad.h"

void TLayerModel::Fill()
{
   this->beginResetModel();
    Lv.clear();
    Cv.clear();
    Lvisible.clear();

    if(orderrow<0) return;
    TPolylineContainer *pc = TCAD::CAD->OrdersList[orderrow]->Layers;
    gc = TCAD::CAD->OrdersList[orderrow];
    while(pc)
    {
     Lv.append(pc->layer);
     Lvisible.append(pc->visible);
     Cv.append(pc->color);
     pc=pc->next;
    }
   this->endResetModel();
}

int TLayerModel::rowCount(const QModelIndex &parent) const
{
   int s = Lv.size();

   return s;
}
int TLayerModel::columnCount(const QModelIndex &parent) const
{
    return 1;
}

QVariant TLayerModel::data(const QModelIndex &index, int role) const
{
    QString unswer;
    switch (role )
    {
      case Qt::DisplayRole:
            if(index.column()==0)
              unswer =Lv[index.row()];
            else
              unswer = Cv[index.row()];
            return QVariant(unswer);
      break;

      case Qt::CheckStateRole:
      return Lvisible[index.row()];
      break;

    }
    return QVariant();
}

//*************************************************
LayersTable::LayersTable()
{
    plm=0;
    verticalHeader()->hide();
    horizontalHeader()->hide();
    verticalHeader()->setDefaultSectionSize ( verticalHeader()->minimumSectionSize () );

    QPalette pal = this->palette();
    pal.setColor(QPalette::Highlight, 0x4466dd );
    this->setPalette(pal); // updated

    connect(this,SIGNAL(doubleClicked(QModelIndex)),this,SLOT(DoubleClicked(QModelIndex)));
}

void LayersTable::DoubleClicked(const QModelIndex &index)
{
    const QModelIndex tmpindex = index;
    TPolylineContainer *pc = TCAD::CAD->OrdersList[LM->orderrow]->Layers;
    int i=0;
    while(pc)
    {
        if(index.row()==i) break;
        i++;
        pc=pc->next;
    }
    if(pc)
    {
        if( pc->visible )
             pc->visible=0;
        else
             pc->visible=1;

        pc->selectable = pc->visible;

        LM->Fill();
        QWidget * viewport = TCAD::CAD->viewport();
        viewport->update();
        this->update();

//        QItemSelectionModel *selectionModel = this->selectionModel();
//        QItemSelection selection(tmpindex,tmpindex);
//        selectionModel->select(selection, QItemSelectionModel::Select);
        this->setCurrentIndex(tmpindex);
    }
}

void LayersTable::MoveSelUp()
{
    QItemSelectionModel *selectionModel = this->selectionModel();
    QModelIndexList indexes = selectionModel->selectedIndexes();
    QModelIndex index;
    foreach(index, indexes) {};
    SelectedRow = index.row();

    TPolylineContainer *pc = LM->gc->FindByNo(SelectedRow);
    int rez = pc->MoveLayerUp();

    if(rez) SelectedRow--;

    LM->Fill();
    QModelIndex index1 = model()->index(SelectedRow, 0);
    this->setCurrentIndex(index1);
//    QItemSelection selection(index1,index1);
//    selectionModel->select(selection, QItemSelectionModel::Select);

}

void LayersTable::MoveSelDown()
{
    QItemSelectionModel *selectionModel = this->selectionModel();
    QModelIndexList indexes = selectionModel->selectedIndexes();
    QModelIndex index;
    foreach(index, indexes) {};
    SelectedRow = index.row();

    TPolylineContainer *pc = LM->gc->FindByNo(SelectedRow);
    int rez = pc->MoveLayerDown();

    if(rez) SelectedRow++;

    LM->Fill();
    QModelIndex index1 = model()->index(SelectedRow, 0);
    this->setCurrentIndex(index1);
//    QItemSelection selection(index1,index1);
//    selectionModel->select(selection, QItemSelectionModel::Select);

}
void LayersTable::MoveSelUpUp()
{
    QItemSelectionModel *selectionModel = this->selectionModel();
    QModelIndexList indexes = selectionModel->selectedIndexes();
    QModelIndex index;
    foreach(index, indexes) {};
    SelectedRow = index.row();

    TPolylineContainer *pc = LM->gc->FindByNo(SelectedRow);
    int rez = pc->MoveLayerUpUp();

    if(rez) SelectedRow=0;

    LM->Fill();
    QModelIndex index1 = model()->index(SelectedRow, 0);
    this->setCurrentIndex(index1);
//    QItemSelection selection(index1,index1);
//    selectionModel->select(selection, QItemSelectionModel::Select);
}

void LayersTable::MoveSelDownDown()
{
    QItemSelectionModel *selectionModel = this->selectionModel();
    QModelIndexList indexes = selectionModel->selectedIndexes();
    QModelIndex index;
    foreach(index, indexes) {};
    SelectedRow = index.row();

    TPolylineContainer *pc = LM->gc->FindByNo(SelectedRow);
    int rez = pc->MoveLayerDownDown();

    if(rez) SelectedRow=LM->rowCount(index)-1;

    LM->Fill();
    QModelIndex index1 = model()->index(SelectedRow, 0);
    this->setCurrentIndex(index1);
//    QItemSelection selection(index1,index1);
//    selectionModel->select(selection, QItemSelectionModel::Select);
}

void LayersTable::keyPressEvent(QKeyEvent *event)
{
    switch(event->key())
    {
      case Qt::Key_Up:
       if(event->modifiers()==Qt::ControlModifier)
            this->MoveSelUp();
       else
              QTableView::keyPressEvent(event);
       break;
      case Qt::Key_Down:
       if(event->modifiers()==Qt::ControlModifier)
            this->MoveSelDown();
       else
              QTableView::keyPressEvent(event);
       break;

     case Qt::Key_Home:
      if(event->modifiers()==Qt::ControlModifier)
           this->MoveSelUpUp();
     break;
     case Qt::Key_End:
      if(event->modifiers()==Qt::ControlModifier)
           this->MoveSelDownDown();
     break;

      case Qt::Key_Space:
        QItemSelectionModel *selectionModel = this->selectionModel();
        QModelIndexList indexes = selectionModel->selectedIndexes();
        QModelIndex index;
        foreach(index, indexes) {};
        this->DoubleClicked(index);
        this->setCurrentIndex(index);
       break;
    }
}

void LayersTable::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
{
    QModelIndexList indexes = selected.indexes();
    QModelIndex index;
    foreach(index, indexes) {};
    int row = index.row();

    if(TCAD::CAD->ListTool==0) return;
    QItemSelectionModel    *selectModel;
    selectModel = TCAD::CAD->ListTool->OrdersTable.selectionModel();
    indexes = selectModel->selectedIndexes();
    int orderrow = indexes[0].row();
    SelectSlavePLContainer( TCAD::CAD->OrdersList[orderrow]->FindByNo(row) );
}

void LayersTable::SelectSlavePLContainer(TPolylineContainer* pc)
{
    if(!plm) return;
    plm->pc = pc;
    plm->Fill();
}

// ****************************************************

void PLineModel::Fill()
{
   this->beginResetModel();
    Lv.clear();
    Lvisible.clear();
    if(!pc) return;

    TPolyline *p = pc->Polylines;
    while(p)
    {
      if(p->type==0)
      {
        Lv.append( QString::number( p->L,0,2 ) );
        Lvisible.append(p->ismus1);
      }
      if(p->type==1)
      {
          Lv.append( "Отв. d="+QString::number( p->cr*2 ) );
      }
     p=p->next;
    }
   this->endResetModel();
}


int PLineModel::rowCount(const QModelIndex &parent) const
{
   int s = Lv.size();

   return s;
}
int PLineModel::columnCount(const QModelIndex &parent) const
{
    return 1;
}

QVariant PLineModel::data(const QModelIndex &index, int role) const
{
    QString unswer;
    switch (role )
    {
      case Qt::DisplayRole:
             unswer =Lv[index.row()];
             return QVariant(unswer);

      break;

      case Qt::CheckStateRole:
//           return Lvisible[index.row()];
      break;

    }
    return QVariant();
}

//********************************************************
PLineTable::PLineTable()
{
    plmodel=0;
//    verticalHeader()->hide();
    horizontalHeader()->hide();
    verticalHeader()->setDefaultSectionSize ( verticalHeader()->minimumSectionSize () );

    CareSelectPolyline=0;

    QPalette pal = this->palette();
    pal.setColor(QPalette::Highlight, 0x4466dd );
    this->setPalette(pal); // updated

    connect(this,SIGNAL(doubleClicked(QModelIndex)),this,SLOT(DoubleClicked(QModelIndex)));
}


void PLineTable::DoubleClicked(const QModelIndex &index)
{
    const QModelIndex tmpindex = index;
    TPolyline *p = plmodel->pc->Polylines;
    int i=0;
    while(p)
    {
        if(index.row()==i) break;
        i++;
        p=p->next;
    }
    if(p)
    {
        if( p->ismus1 )
             p->ismus1=0;
        else
             p->ismus1=1;

        plmodel->Fill();
        QWidget * viewport = TCAD::CAD->viewport();
        viewport->update();
        this->update();

        QItemSelectionModel *selectionModel = this->selectionModel();
        QItemSelection selection(tmpindex,tmpindex);
        selectionModel->select(selection, QItemSelectionModel::Select);
    }
}

void PLineTable::keyPressEvent(QKeyEvent *event)
{
    switch(event->key())
    {
      case Qt::Key_Up:
       if(event->modifiers()==Qt::ControlModifier)
            this->MoveSelUp();
       else
              QTableView::keyPressEvent(event);
       break;
      case Qt::Key_Down:
       if(event->modifiers()==Qt::ControlModifier)
            this->MoveSelDown();
       else
              QTableView::keyPressEvent(event);
       break;

     case Qt::Key_Home:
      if(event->modifiers()==Qt::ControlModifier)
           this->MoveSelUpUp();
     break;
     case Qt::Key_End:
      if(event->modifiers()==Qt::ControlModifier)
           this->MoveSelDownDown();
     break;


      case Qt::Key_Space:

        QItemSelectionModel *selectionModel = this->selectionModel();
        QModelIndexList indexes = selectionModel->selectedIndexes();
        QModelIndex index;
        foreach(index, indexes) {};
        this->DoubleClicked(index);
        this->setCurrentIndex(index);
       break;
    }
}

void PLineTable::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
{
   if(CareSelectPolyline) return;
   if(!plmodel) return;

    QModelIndexList indexes = selected.indexes();
    QModelIndex index;
    foreach(index, indexes) {};
    int row = index.row();

    TPolyline *p = plmodel->pc->FindPolylineByNoInLayer(row);
    TCAD::CAD->ClearAllSelectionsOnLayers();
    if(p) p->setSelected(1);
}


void PLineTable::MoveSelUp()
{
    QItemSelectionModel *selectionModel = this->selectionModel();
    QModelIndexList indexes = selectionModel->selectedIndexes();
    QModelIndex index;
    foreach(index, indexes) {};
    SelectedRow = index.row();

    TPolyline *p = plmodel->pc->FindPolylineByNoInLayer(SelectedRow);
    int rez = p->MovePlUp();

    if(rez) SelectedRow--;

    QModelIndex index1 = model()->index(SelectedRow, 0);
    this->setCurrentIndex(index1);
}

void PLineTable::MoveSelDown()
{
    QItemSelectionModel *selectionModel = this->selectionModel();
    QModelIndexList indexes = selectionModel->selectedIndexes();
    QModelIndex index;
    foreach(index, indexes) {};
    SelectedRow = index.row();

    TPolyline* p = plmodel->pc->FindPolylineByNoInLayer(SelectedRow);
    int rez = p->MovePlDown();

    if(rez) SelectedRow++;

    QModelIndex index1 = model()->index(SelectedRow, 0);
    this->setCurrentIndex(index1);

}
void PLineTable::MoveSelUpUp()
{
    QItemSelectionModel *selectionModel = this->selectionModel();
    QModelIndexList indexes = selectionModel->selectedIndexes();
    QModelIndex index;
    foreach(index, indexes) {};
    SelectedRow = index.row();

    TPolyline *p = plmodel->pc->FindPolylineByNoInLayer(SelectedRow);
    int rez = p->MovePlUpUp();

    if(rez) SelectedRow=0;

    QModelIndex index1 = model()->index(SelectedRow, 0);
    this->setCurrentIndex(index1);
}

void PLineTable::MoveSelDownDown()
{
    QItemSelectionModel *selectionModel = this->selectionModel();
    QModelIndexList indexes = selectionModel->selectedIndexes();
    QModelIndex index;
    foreach(index, indexes) {};
    SelectedRow = index.row();

    TPolyline *p = plmodel->pc->FindPolylineByNoInLayer(SelectedRow);
    int rez = p->MovePlDownDown();

    if(rez) SelectedRow=plmodel->rowCount(index)-1;

    QModelIndex index1 = model()->index(SelectedRow, 0);
    this->setCurrentIndex(index1);
}

//*****************************************************
TListTool::TListTool()
{
    QVBoxLayout *Layout_top = new QVBoxLayout(this);

    QHBoxLayout *LayoutThick = new QHBoxLayout();
    Layout_top->addLayout(LayoutThick);
    QLabel *l = new QLabel("Толщина материала");
    LayoutThick->addWidget(l);
    LayoutThick->addSpacing(30);
    LayoutThick->addWidget(&ThickBase);
    LayoutThick->addStretch();
    QStringList sl;
    sl<<"1мм"<<"2мм"<<"3мм"<<"4мм";
    ThickBase.addItems(sl);

    QPushButton *lup = new QPushButton("<");
    QPushButton *ldown = new QPushButton(">");
    LayoutThick->addWidget(lup);
    LayoutThick->addWidget(ldown);
    lup->setMinimumWidth(20);
    ldown->setMinimumWidth(20);
    connect(lup, SIGNAL(clicked(bool)), this, SLOT(OrderUp(bool)));
    connect(ldown, SIGNAL(clicked(bool)), this, SLOT(OrderDown(bool)));


    QVBoxLayout *LayoutLeft = new QVBoxLayout();
    QHBoxLayout *HLtop = new QHBoxLayout();

    HLtop->addLayout(LayoutLeft);

    QVBoxLayout *Layout = new QVBoxLayout();
    HLtop->addLayout(Layout);

    Layout_top->addWidget(&OrdersTable);
    OrdersTable.setSelectionMode(QAbstractItemView::SingleSelection);
    OrdersTable.setSelectionBehavior(QAbstractItemView::SelectRows);
    OrdersTable.setStyleSheet("QTableView:item:selected {background-color: #4444aa; color: #FFFFFF}\n"
                                 "QTableView:item:selected:focus {background-color: #4444aa;}");
    OrdersTable.setMaximumHeight(200);

    Layout_top->addLayout(HLtop);

    QHBoxLayout *HL = new QHBoxLayout();
    LayoutLeft->addLayout(HL);

    QPushButton *btn = new QPushButton("<<");
    btn->setStyleSheet("QPushButton { padding: 1px;}");
    HL->addWidget(btn);
    connect(btn,SIGNAL(clicked(bool)),&Layers,SLOT(MoveSelUpUp()));
    btn->setToolTip("Переместить в начало CTRL-Home");

    btn = new QPushButton("<");
    btn->setStyleSheet("QPushButton { padding: 1px;}");
    HL->addWidget(btn);
    connect(btn,SIGNAL(clicked(bool)),&Layers,SLOT(MoveSelUp()));
    btn->setToolTip("Переместить выше CTRL-вверх");

    btn = new QPushButton(">");
    btn->setStyleSheet("QPushButton { padding: 1px;}");
    HL->addWidget(btn);
    connect(btn,SIGNAL(clicked(bool)),&Layers,SLOT(MoveSelDown()));
    btn->setToolTip("Переместить ниже CTRL-вниз");

    btn = new QPushButton(">>");
    btn->setStyleSheet("QPushButton { padding: 1px;}");
    HL->addWidget(btn);
    connect(btn,SIGNAL(clicked(bool)),&Layers,SLOT(MoveSelDownDown()));
    btn->setToolTip("Переместить в конец CTRL-End");

    LayoutLeft->addWidget(&Layers);
    Layers.setMaximumWidth(120);
    Layers.setMinimumWidth(120);
    Layers.setModel(&LayersModel);
    Layers.LM = &LayersModel;

    Layers.setColumnWidth(0,110);
    Layers.setSelectionBehavior(QAbstractItemView::SelectRows);
    Layers.setSelectionMode(QAbstractItemView::SingleSelection);

    HL = new QHBoxLayout();
    Layout->addLayout(HL);

    btn = new QPushButton("<<");
    btn->setStyleSheet("QPushButton { padding: 1px;}");
    HL->addWidget(btn);
    connect(btn,SIGNAL(clicked(bool)),&PLT,SLOT(MoveSelUpUp()));
    btn->setToolTip("Переместить в начало CTRL-Home");

    btn = new QPushButton("<");
    btn->setStyleSheet("QPushButton { padding: 1px;}");
    HL->addWidget(btn);
    connect(btn,SIGNAL(clicked(bool)),&PLT,SLOT(MoveSelUp()));
    btn->setToolTip("Переместить выше CTRL-вверх");

    btn = new QPushButton(">");
    btn->setStyleSheet("QPushButton { padding: 1px;}");
    HL->addWidget(btn);
    connect(btn,SIGNAL(clicked(bool)),&PLT,SLOT(MoveSelDown()));
    btn->setToolTip("Переместить ниже CTRL-вниз");

    btn = new QPushButton(">>");
    btn->setStyleSheet("QPushButton { padding: 1px;}");
    HL->addWidget(btn);
    connect(btn,SIGNAL(clicked(bool)),&PLT,SLOT(MoveSelDownDown()));
    btn->setToolTip("Переместить в конец CTRL-end");


    PLT.setModel(&PLm);
    PLT.plmodel=&PLm;
    PLT.setSelectionBehavior(QAbstractItemView::SelectRows);
    PLT.setSelectionMode(QAbstractItemView::SingleSelection);
    Layers.plm = &PLm;
    Layout->addWidget(&PLT);

    connect(btn,SIGNAL(clicked(bool)),this,SLOT(RefreshLayers()));

    sortHmenu = new QAction(tr("Горизонтальные вперёд"), this);
    sortVmenu = new QAction(tr("Вертикальные вперёд"), this);
    sortSqaremenu = new QAction(tr("Упорядочить"), this);
    calcLenMenu = new QAction(tr("Посчитать длину всего"), this);
//            ReversePolylineAct->setStatusTip(tr("Разворот полилинии"));
    connect(sortHmenu, SIGNAL(triggered()), this, SLOT(DoSortH()));
    connect(sortVmenu, SIGNAL(triggered()), this, SLOT(DoSortV()));
    connect(sortSqaremenu, SIGNAL(triggered()), this, SLOT(DoSortSqare()));
    connect(calcLenMenu, SIGNAL(triggered()), this, SLOT(DoCalcLength()));

    connect(&OrdersTable, SIGNAL(currentCellChanged(int,int,int,int)), this, SLOT(OrderSelected(int,int,int,int)));
    connect(&ThickBase, SIGNAL(currentIndexChanged(int)), this, SLOT(ThickSelected(int)));
}

void TListTool::RefreshLayers()
{
    //LayersModel.Fill();
    //Layers.model()->submit();
}

void TListTool::DisplayList()
{
   Layers.selectRow(0);
}

void TListTool::OrderSelected(int currRow, int currCol, int, int)
{
   LayersModel.orderrow = currRow;
   LayersModel.Fill();
}

void TListTool::SelectPolyline(TPolyline* _p)
{
  PLT.CareSelectPolyline=1;
  int i;

  for(int k=0; k<TCAD::CAD->OrdersList.count(); k++)
  {
   TPolylineContainer *pc = TCAD::CAD->OrdersList[k]->Layers;
    i=0;
    while(pc)
    {
        if(_p->plcontainer == pc) break;
        i++;
        pc = pc->next;
    }
    if(pc)
    {
        OrdersTable.selectRow(k);
        QModelIndex index1 = LayersModel.index(i, 0);
        Layers.setCurrentIndex(index1);

    }
  }

    TPolyline *p = _p->plcontainer->Polylines;
    i=0;
    while(p)
    {
        if(p==_p) break;
        i++;
        p=p->next;
    }

    PLm.pc = _p->plcontainer;
    PLm.Fill();
    QModelIndex index1 = PLm.index(i, 0);
    PLT.setCurrentIndex(index1);
  PLT.CareSelectPolyline=0;
}

void TListTool::ThickSelected(int row)
{
  TCAD::CAD->curr_board_thick=row;
}

void TListTool::FillOrderTable()
{
  OrdersTable.clear();
  int n = OrdersTable.rowCount();
  for( int i = 0; i < n; i++ ) OrdersTable.removeRow( 0 );

  QStringList headers;
  headers << "id"<<"№"<<"Название"<<"ф"<<"ножи"<<"длина";
  OrdersTable.verticalHeader()->setVisible(false);
  OrdersTable.setColumnCount( 6 );
  OrdersTable.setColumnHidden(0,true);
  OrdersTable.setColumnWidth(1, 65 );
  OrdersTable.setColumnWidth(2, 100 );
  OrdersTable.setColumnWidth(3, 30 );
  OrdersTable.setColumnWidth(4, 50 );
  OrdersTable.setColumnWidth(5, 50 );
  OrdersTable.setHorizontalHeaderLabels(headers);

  for(int i=0; i<TCAD::CAD->OrdersList.count(); i++)
  {
        OrdersTable.insertRow(i);
        OrdersTable.setRowHeight(i, 20);
        OrdersTable.setItem(i,0, new QTableWidgetItem( TCAD::CAD->OrdersList[i]->orderinfo.idorder) );
        OrdersTable.setItem(i,1, new QTableWidgetItem(TCAD::CAD->OrdersList[i]->orderinfo.zak_number));
        OrdersTable.setItem(i,2, new QTableWidgetItem(TCAD::CAD->OrdersList[i]->orderinfo.CName));
        OrdersTable.setItem(i,3, new QTableWidgetItem( TCAD::CAD->OrdersList[i]->orderinfo.base_thickness) );
        OrdersTable.setItem(i,4, new QTableWidgetItem( TCAD::CAD->OrdersList[i]->orderinfo.blade_thickness) );
        OrdersTable.setItem(i,5, new QTableWidgetItem( TCAD::CAD->OrdersList[i]->orderinfo.blade_len) );
  }
  if(OrdersTable.rowCount())
        OrdersTable.selectRow(0);
  int curr_thickindex = 3;
  int th = TCAD::CAD->OrdersList.first()->orderinfo.base_thickness.toInt();
  if(th==18) curr_thickindex = 0;
  if(th==15) curr_thickindex = 1;
  if(th==12) curr_thickindex = 2;
  ThickBase.setCurrentIndex(curr_thickindex);
  TCAD::CAD->curr_board_thick = curr_thickindex;
}

void TListTool::OrderUp(bool)
{
  if(LayersModel.orderrow>0)
  {
        int row = LayersModel.orderrow;
        TCAD::CAD->OrdersList.swap(LayersModel.orderrow, LayersModel.orderrow-1);
        FillOrderTable();
        OrdersTable.selectRow(row-1);
  }
}
void TListTool::OrderDown(bool)
{
  if(LayersModel.orderrow < TCAD::CAD->OrdersList.count()-1)
  {
        int row = LayersModel.orderrow;
        TCAD::CAD->OrdersList.swap(LayersModel.orderrow, LayersModel.orderrow+1);
        FillOrderTable();
        OrdersTable.selectRow(row+1);
  }
}

void TListTool::contextMenuEvent(QContextMenuEvent *event)
{
    QMenu menu(this);
//    menu.addAction(sortHmenu);
//    menu.addAction(sortVmenu);
    menu.addAction(sortSqaremenu);
    menu.addAction(calcLenMenu);
    menu.exec(event->globalPos());
}


void TListTool::DoSortH()
{

}

void TListTool::DoSortV()
{

}

void TListTool::DoSortSqare()
{

//    TCAD::CAD->GC->OptimizeAll();
    return;
/*
    TPolylineContainer *pc = PLm.pc;
    if(!pc) return;
    if(!pc->Polylines) return;
    TPolyline *p = pc->Polylines;
    TPolyline *pmin=pc->Polylines;
    p=p->next;
    while(p)
    {
        if(p->firstV->x < pmin->firstV->x)
        p=p->next;
    }

*/

}

void TListTool::DoCalcLength()
{
    double sum=0;
    int cnt=0;
    TPolylineContainer *pc = TCAD::CAD->OrdersList.first()->Layers;
    while(pc)
    {
        sum += pc->CalcLenLay();
        cnt +=pc->CalcCntLay();
        pc=pc->next;
    }

    QMessageBox* pmbx = new QMessageBox("Длина","Длина всех примитивов: "+QString::number(sum,3,1)+"\nКоличество врезок: "+QString::number(cnt),
                        QMessageBox::Information, QMessageBox::Ok, QMessageBox::NoButton,
                        QMessageBox::NoButton | QMessageBox::Escape);
    pmbx->exec();
    delete pmbx;
}





