#ifndef TPOLYLINES_H
#define TPOLYLINES_H

#include <qgraphicsitem.h>
#include <QGraphicsView>
#include <QGraphicsPathItem>
#include<QPen>
#include <QtMath>
#include<QBrush>
#include<QPainter>

class TCAD;
class TPolylineContainer;

class TVertex
{
 public:
  double x,y;
  double len;
  double bulge;
  double a1,a2; // угол от центра окружности до первой и следующей вершины
  double x0,y0,r; // центр и радиус
  TVertex *next;
  TVertex *prev;
  TVertex *last();

  TVertex();
  explicit TVertex( double x,  double y,  double b);
  TVertex  operator=(TVertex );
  double LenToPoint( double x,  double y);
  double TouchBridgeOnSegment( double xx,  double yy);

  static double Xmatrix(double X);
  static double Ymatrix(double _Y);

  TVertex halfline();

};

class TPolyline : public QGraphicsPathItem
{
  private:

public:
  static double dot_shift;
  TPolylineContainer *plcontainer;

  int type;  // 0 - polyline 1 - circle
  int color;
  double cx,cy,cr; // для отверстий

  int is_copy;
  int make_cnt;

  double x_min, y_min, x_max, y_max;

  TVertex *firstV;
  TPolyline *next;
  TPolyline *prev;
  TPolyline *last();
  TVertex *lastV();
  QPainterPath *path;
  QPainterPath *Vpath; // для отрисовки сегмента
  TVertex *onMouseV;
  TVertex *SelectedVertex;
  TVertex *onMouseSegment;
  TVertex *SelectedSegment;
  TVertex onMousePoint;

  TPolyline *PartContour;
  void SetLayerColor(QString _color);

  int tryConcatError; // Мигать при ошибке соединения с другой полилинией

  double elong1, elong2;
  int ismus1, ismus2;

  double L;

  TPolyline();
  ~TPolyline();
  void Copy(TPolyline*);
  TVertex* AddVertex(double x, double y, double b);
  void AddArc(double x, double y, double r, double a1, double a2);
  void InsertVertex(TVertex *start, TVertex *);
  void CalcBulge(TVertex* v1, TVertex* v2);
  double CalcLength();
  TVertex* FindVertex(double xx, double yy, double r);
  void PathRepair();
  void SetVpath(TVertex *v);
  void CalcMinMax();

  void FindIntersectionsShorten(double *b, double *e);
  void _FindIntersectionsShorten(double *b, double *e);

  TVertex* TouchSegment(double xx, double yy, double _r);
  void Concat(TPolyline *, int);
  void Reverse();

  void onMouseNearestVertex(double x, double y, double r);
  double LengthofNewVertex();
  int ApplyNewVertex(TVertex*, double);
  void close();

  void setSelected(bool selected);
  int isCircle();
  int isPolygon();
  void cmdCI();
  static void cmdCCI();

  double VectorAngle(int is_first);

  int MovePlUp();
  int MovePlDown();
  int MovePlUpUp();
  int MovePlDownDown();


  void hoverMoveEvent_nested(QPointF m);
protected:
  void paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget);
 // void hoverEnterEvent(QGraphicsSceneHoverEvent * event);
  void hoverMoveEvent(QGraphicsSceneHoverEvent * event);
  void hoverLeaveEvent ( QGraphicsSceneHoverEvent * event );
  void mousePressEvent(QGraphicsSceneMouseEvent *);

public slots:

};

#define ABS(A) ((A) >= 0 ? (A) : -(A))


double VectorToRadial(double x1, double y1, double x2, double y2);




#endif // TPOLYLINES_H
