#include "tpolylinecontainer.h"
#include"tcad.h"
#include<ui_mainwindow.h>

TPolylineContainer::TPolylineContainer(QString _layer, QString _color, TGraphContainer* _gc)
{
  Polylines=0;
  next=prev=0;
  layer=_layer;
  color = _color;
  visible=1;
  selectable=0;
  preview_selected=0;
  preview_hover=0;
  Type=0;
  GC = _gc;
  outer.setRect(0,0,0,0);
}

TPolylineContainer::~TPolylineContainer()
{
    TPolyline *p = Polylines;
    while(p)
    {
        TPolyline*p1=p;
        p=p->next;
        delete p1;
    }
}

TPolyline *TPolylineContainer::AddPolyline(TPolyline* newptr)
{
    newptr->plcontainer=this;
    if(Polylines==0)
    {
        Polylines=newptr;
        return newptr;
    } else
    {
        TPolyline *ptr=Polylines->last();
        ptr->next=newptr;
        newptr->prev=ptr;
        return newptr;
    }
}

TPolyline *TPolylineContainer::NewPolyline()
{
  TPolyline *newptr = new TPolyline();
  return AddPolyline(newptr);
}

void TPolylineContainer::delPolyline(TPolyline*p)
{
    if(p->prev) p->prev->next=p->next;
    else
        p->plcontainer->Polylines=p->next;
    if(p->next) p->next->prev=p->prev;
    GC->CAD->dropAnySelectionFromPolyline(p);
    delete p;
}



TPolyline * TPolylineContainer::FindPolyline(qreal x, qreal y, qreal r)
{
   TPolyline *p=Polylines;
   while(p)
   {
       if( p->TouchSegment(x,y,r) )
        return p;
       p=p->next;
   }
 return 0;
}


void TPolylineContainer::MergePolylines()
{
 if( !Polylines) return;
    TPolyline *p1, *ptmp;
    TPolyline *p = Polylines;
    while(p)
    {
        p1=p->next;
        while(p1)
        {
            ptmp = p1->next;
            GC->CAD->Concat(p,p1);
            p1=ptmp;
        }

        p=p->next;
    }
 return;
}


void TPolylineContainer::CreateOffsetLines(TPolyline* p)
{
   double w = 0.71/2;
//   if(TCAD::CAD->bender) w = TCAD::CAD->bender->rulewidth/2;

   TPolyline* np=0;
    TVertex *v = p->firstV;
    while(v->next)
    {
        double a = - VectorToRadial(v->x,v->y, v->next->x,v->next->y);
        double aa1 = 0 ;
        if( v->bulge != 0 )
           aa1 = 2*atan(v->bulge);

        double bx1= v->x+w*cos( a+aa1+M_PI/2 ) ;
        double by1= v->y-w*sin( a+aa1+M_PI/2 ) ;
        double bx2= v->next->x+w*cos( a-aa1+M_PI/2 ) ;
        double by2= v->next->y-w*sin( a-aa1+M_PI/2 ) ;

        double ex1= v->x+w*cos( a+aa1-M_PI/2 ) ;
        double ey1= v->y-w*sin( a+aa1-M_PI/2 ) ;
        double ex2= v->next->x+w*cos( a-aa1-M_PI/2 ) ;
        double ey2= v->next->y-w*sin( a-aa1-M_PI/2 ) ;


        np = NewPolyline();
        np->AddVertex(bx1,by1,v->bulge);
        np->AddVertex(bx2,by2,0);
        np = NewPolyline();
        np->AddVertex(ex1,ey1,v->bulge);
        np->AddVertex(ex2,ey2,0);

        v = v->next;
    }
}

void TPolylineContainer::CreateOffsetLinesLeft(TPolyline* p)
{
    double w = 0.71/2;
     TPolyline* np=0;
     TVertex *v = p->firstV;
     while(v->next)
     {
         double a = - VectorToRadial(v->x,v->y, v->next->x,v->next->y);
         double aa1 = 0 ;
         if( v->bulge != 0 )
            aa1 = 2*atan(v->bulge);

         double bx1= v->x+w*cos( a+aa1+M_PI/2 ) ;
         double by1= v->y-w*sin( a+aa1+M_PI/2 ) ;
         double bx2= v->next->x+w*cos( a-aa1+M_PI/2 ) ;
         double by2= v->next->y-w*sin( a-aa1+M_PI/2 ) ;

//         double ex1= v->x+w*cos( a+aa1-M_PI/2 ) ;
//         double ey1= v->y-w*sin( a+aa1-M_PI/2 ) ;
//         double ex2= v->next->x+w*cos( a-aa1-M_PI/2 ) ;
//         double ey2= v->next->y-w*sin( a-aa1-M_PI/2 ) ;

         np = NewPolyline();
         np->AddVertex(bx1,by1,v->bulge);
         np->AddVertex(bx2,by2,0);
//         np = NewPolyline();
//         np->AddVertex(ex1,ey1,v->bulge);
//         np->AddVertex(ex2,ey2,0);

         v = v->next;
     }
}
void TPolylineContainer::CreateOffsetLinesRight(TPolyline* p)
{
    double w = 0.71/2;
     TPolyline* np=0;
     TVertex *v = p->firstV;
     while(v->next)
     {
         double a = - VectorToRadial(v->x,v->y, v->next->x,v->next->y);
         double aa1 = 0 ;
         if( v->bulge != 0 )
            aa1 = 2*atan(v->bulge);

//         double bx1= v->x+w*cos( a+aa1+M_PI/2 ) ;
//         double by1= v->y-w*sin( a+aa1+M_PI/2 ) ;
//         double bx2= v->next->x+w*cos( a-aa1+M_PI/2 ) ;
//         double by2= v->next->y-w*sin( a-aa1+M_PI/2 ) ;

         double ex1= v->x+w*cos( a+aa1-M_PI/2 ) ;
         double ey1= v->y-w*sin( a+aa1-M_PI/2 ) ;
         double ex2= v->next->x+w*cos( a-aa1-M_PI/2 ) ;
         double ey2= v->next->y-w*sin( a-aa1-M_PI/2 ) ;

//         np = NewPolyline();
//         np->AddVertex(bx1,by1,v->bulge);
//         np->AddVertex(bx2,by2,0);
         np = NewPolyline();
         np->AddVertex(ex1,ey1,v->bulge);
         np->AddVertex(ex2,ey2,0);

         v = v->next;
     }
}

void TPolylineContainer::ToScene( QGraphicsScene *_scene)
{
    TPolyline *p = Polylines;
    while(p)
    {
       _scene->addItem(p);
        p = p->next;
    }
}

int TPolylineContainer::MoveLayerUp()
{
    if(this==0) return 1;

    TPolylineContainer * pc = this->prev;
    if(!prev) return 0;

    prev->next=next;
    if(next) next->prev=prev;

    if( prev->prev )
    {
        prev->prev->next = this;
    }
    else
        GC->Layers = this;
    this->prev= prev->prev;
    next = pc;
    pc->prev=this;

    return 1;
}

int TPolylineContainer::MoveLayerUpUp()
{
    TPolylineContainer * pc = GC->Layers;
    if(pc==this) return 0;

    prev->next=next;
    if(next) next->prev=prev;

    pc->prev = this;
    this->next = pc;
    this->prev=0;

    GC->Layers = this;

    return 1;
}

int TPolylineContainer::MoveLayerDown()
{
    TPolylineContainer * pc = this->next;
    if(!pc) return 0;
    if(pc->next) pc->next->prev=this;
    next->prev=prev;
    if(prev) prev->next=pc;
     else GC->Layers = pc;
    this->prev = pc;
    this->next=0;
    if(pc)
     {
        this->next = pc->next;

        pc->next=this;
     }

   return 1;
}

int TPolylineContainer::MoveLayerDownDown()
{
    TPolylineContainer * pc = this;

    if(GC->Layers==this) GC->Layers=this->next;
    while(pc->next) pc=pc->next;
    if(pc==this) return 0;

    next->prev=prev;
    if(prev) prev->next=next;

    pc->next=this;
    this->prev = pc;
    this->next=0;

   return 1;
}


TPolyline* TPolylineContainer::FindPolylineByNoInLayer(int No)
{
    if(No==-1) return 0;
    TPolyline *p = Polylines;
    int i=0;
    while(p)
    {
        if(i==No) return p;
        i++;
        p=p->next;
    }
   return 0;
}

double TPolylineContainer::distForInline( double x, double y, TPolyline* curr_p, TPolyline *p, int is_last)
{
   double ret;

   if(is_last==0)
   {
    if( curr_p )
    {
        ret = LineLen(curr_p->lastV()->x, curr_p->lastV()->y, p->firstV->x,p->firstV->y);
        double v1 = curr_p->VectorAngle(1);
        double xx1 = curr_p->lastV()->x;
        double yy1 = curr_p->lastV()->y;
        double v2 = VectorToRadial(xx1, yy1, p->firstV->x, p->firstV->y);
        double da = qAbs( v1 - v2 );
        if(da>M_PI/2) da=qAbs(M_PI-da);
        if( da > 0.1 )
        {
            ret +=7 + 10*da;
        }
    } else
    {
        ret = LineLen(x, y, p->firstV->x,p->firstV->y);
    }
   } else
   {
    if( curr_p )
    {
        ret = LineLen(curr_p->lastV()->x, curr_p->lastV()->y, p->lastV()->x,p->lastV()->y);
        double v1 = curr_p->VectorAngle(1);
        double xx1 = curr_p->lastV()->x;
        double yy1 = curr_p->lastV()->y;
        double v2 = VectorToRadial(xx1, yy1, p->lastV()->x, p->lastV()->y);
        double da = qAbs( v1 - v2 );
        if(da>M_PI/2) da=qAbs(M_PI-da);
        if( da > 0.1 )
        {
            ret +=7 + 10*da;
        }
    } else
    {
        ret = LineLen(x, y, p->firstV->x,p->firstV->y);
    }

   }
   return ret;
}

TPolyline* TPolylineContainer::FindNearestVertex(TPolyline* firstP, double x, double y, TPolyline *PartCont, TPolyline* curr_p)
{
   double r=1e25;
   double r_last0=1e25;
   double r_last1=1e25;
   double tmpr;
   double da;
    TPolyline *pfound=0;
    TPolyline *pfound_inverse=0;

    TPolyline *p = firstP;
    while (p) {
       if(p->PartContour==PartCont)
       {
            if( (tmpr = distForInline( x,y, curr_p, p, 0 )) < r )
            {
                r=tmpr;
                r_last0=r;
                pfound=p;
            }
            if( (tmpr = distForInline( x,y, curr_p, p, 1 )) < r )
            {
                r=tmpr;
                r_last1=r;
                pfound_inverse=p;
            }
       }
       p=p->next;
    }

    if(r_last1<r_last0)
    {
       pfound_inverse->Reverse();
       return pfound_inverse;
    }

  return pfound;
}

void TPolylineContainer::MoveP(TPolyline *src, TPolyline* dest)
{
    if(src==dest) return;

    TPolyline *tmp1 = src->next;
    TPolyline *tmp2 = src->prev;

    if(dest->prev) dest->prev->next = src;
    src->prev = dest->prev;
    dest->prev = src;
    src->next=dest;

    if(tmp1) tmp1->prev = tmp2;
    if(tmp2) tmp2->next = tmp1;
}


double TPolylineContainer::CalcLenLay()
{
    double sum=0;
    TPolyline *p = this->Polylines;
    while(p)
    {
      sum+= p->L;
      p=p->next;
    }
    return sum;
}
int TPolylineContainer::CalcCntLay()
{
    int cnt=0;
    TPolyline *p = this->Polylines;
    while(p)
    {
      cnt++;
      p=p->next;
    }
    return cnt;
}


void TPolylineContainer::Rotate(int steps)
{
    if( steps==1)
    {
      TPolyline *p = Polylines;
      while(p)
      {
            double tmp1x = p->cx;
            p->cx = - p->cy;
            p->cy = tmp1x;

            TVertex *v = p->firstV;
            while(v)
            {
                double tmpx = v->x;
                v->x = - v->y;
                v->y = tmpx;
                v=v->next;
            }
            p->CalcLength();
            p->PathRepair();

            p=p->next;
      }
    }

    if( steps==2)
    {
      TPolyline *p = Polylines;
      while(p)
      {
            p->cx = - p->cx;
            p->cy = - p->cy;

            TVertex *v = p->firstV;
            while(v)
            {
                v->x = - v->x;
                v->y = - v->y;
                v=v->next;
            }
            p->CalcLength();
            p->PathRepair();

            p=p->next;
      }
    }

    if( steps==3)
    {
      TPolyline *p = Polylines;
      while(p)
      {
            double tmp1x = p->cx;
            p->cx =  p->cy;
            p->cy = -tmp1x;

            TVertex *v = p->firstV;
            while(v)
            {
                double tmpx = v->x;
                v->x =  v->y;
                v->y = -tmpx;
                v=v->next;
            }
            p->CalcLength();
            p->PathRepair();

            p=p->next;
      }
    }
    qDebug()<<CalcZeroOffset();
}

QPointF TPolylineContainer::CalcZeroOffset()
{
    QPointF z(10e10,10e10);
    TPolyline *p = Polylines;
    if(p)
    {
          while(p)
          {
            p->CalcMinMax();
            if(p->x_min < z.x()) z.setX(p->x_min);
            if(p->y_min < z.y()) z.setY(p->y_min);
            p=p->next;
          }
    }
    return z;
}

void TPolylineContainer::CalcOutsize()
{
    QPointF z(10e10,10e10);
    QPointF z1(-10e10,-10e10);
    TPolyline *p = Polylines;
    if(p)
    {
          while(p)
          {
            p->CalcMinMax();
            if(p->x_min < z.x()) z.setX(p->x_min);
            if(p->y_min < z.y()) z.setY(p->y_min);
            if(p->x_max > z1.x()) z1.setX(p->x_max);
            if(p->y_max > z1.y()) z1.setY(p->y_max);
            p=p->next;
          }
    }
    outer.setBottom(z.y());
    outer.setTop(z1.y());
    outer.setLeft(z.x());
    outer.setRight(z1.x());
}

double TPolylineContainer::CalcSquare()
{
    return outer.width()*outer.height();
}

void TPolylineContainer::Move(double _x, double _y)
{
    TPolyline *p = Polylines;
    if(p)
    {
          while(p)
          {
            TVertex *v = p->firstV;
            while(v)
            {
                v->x+=_x;
                v->y+=_y;
                v->x0+= _x;
                v->y0+= _y;
                v=v->next;
            }
            p->PathRepair();
            p=p->next;
          }
    }
    CalcOutsize();
}
