#include "tppconfig.h"
#include "ui_tppconfig.h"

TppConfig::TppConfig(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::TppConfig)
{
    ui->setupUi(this);
    QString dir = qApp->applicationDirPath();
    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
    setWindowTitle("Настройки генератора G-кода");
    iniconf = new QSettings(dir+"/qmcam.ini", QSettings::IniFormat);
    loadini();
}

TppConfig::~TppConfig()
{
    delete ui;
}

void TppConfig::on_pushButton_2_clicked()
{
    QString fileName = QFileDialog::getSaveFileName(this, "Save G-code file", dir_output, "(*.ngc)" );

    if(fileName=="") return ;
    dir_output = fileName;
    ui->OutputFilePath->setText(fileName);
    return;
}

QString TppConfig::get_last_opened_file()
{
    last_opened_file = iniconf->value("CONF/LAST_OPENED_FILE", "open.dxf").toString();
    return last_opened_file;
}

void TppConfig::set_last_opened_file(QString filename)
{
    iniconf->setValue("CONF/LAST_OPENED_FILE", filename);
    last_opened_file = filename;
}


void TppConfig::loadini()
{
    QString prefix="";
    dir_output = iniconf->value("CONF/DIR_OUTPUT","output.ngc").toString();
    first_text = iniconf->value("GGEN/START_TEXT","G90").toString();
    tool_engage = iniconf->value("GGEN/TOOL_ON","M64 P0").toString();
    tool_off = iniconf->value("GGEN/TOOL_OFF","M65 P0").toString();
    prog_end = iniconf->value("GGEN/PROG_END","M30\n;").toString();
    MLS = iniconf->value("GGEN/MLS","10").toDouble();
    MLSinc = iniconf->value("GGEN/MLSinc","5").toInt();

    t3bridges = iniconf->value("GGEN/P3bridges"+prefix,"M101 P3\nM105 P3\n").toString();
    t3bridgesF = iniconf->value("GGEN/PF3bridges"+prefix,"800").toString();
    use_3pt_briges = iniconf->value("GGEN/Use3bridges"+prefix,"0").toInt();

    ui->OutputFilePath->setText(dir_output);
    ui->textEditProlog->setText(first_text);
    ui->textEditEpilog->setText(prog_end);
    ui->textEditCutOn->setText(tool_engage);
    ui->textEditCutOff->setText(tool_off);
    ui->doubleSpinBox_PieLen->setValue(MLS);
    ui->spinBox_PieFIncrease->setValue(MLSinc);

    for (int i = 0; i < 4; ++i)
    {
        switch (i) {
        case 0: prefix="18"; break;
        case 1: prefix="15"; break;
        case 2: prefix="12"; break;
        case 3: prefix="10"; break;
        default:   prefix="def"; break;
        }

        ttext[i] = iniconf->value("GGEN/Ptext"+prefix,"M101 P1\nM105 P3\n").toString();
        t1[i] = iniconf->value("GGEN/P1"+prefix,"M101 P1\nM105 P2\n").toString();
        t2[i] = iniconf->value("GGEN/P2"+prefix,"M101 P2\nM105 P2\n").toString();
        t3[i] = iniconf->value("GGEN/P3"+prefix,"M101 P3\nM105 P5\n").toString();
        t4[i] = iniconf->value("GGEN/P4"+prefix,"M101 P4\nM105 P2\n").toString();
        tcut[i] = iniconf->value("GGEN/Pcut"+prefix,"M101 P1\nM105 P1\n").toString();

        ttextF[i] = iniconf->value("GGEN/PFtext"+prefix,"2000").toString();
        t1F[i] = iniconf->value("GGEN/PF1"+prefix,"1000").toString();
        t2F[i] = iniconf->value("GGEN/PF2"+prefix,"900").toString();
        t3F[i] = iniconf->value("GGEN/PF3"+prefix,"800").toString();
        t4F[i] = iniconf->value("GGEN/PF4+prefix","700").toString();
        tcutF[i] = iniconf->value("GGEN/PFcut"+prefix,"1400").toString();
    }

    ui->plainTextEdit13bridges_18->setPlainText(t3bridges);
    ui->spinBoxF13bridges_18->setValue( t3bridgesF.toLong() ) ;
    ui->checkBox33ptBridges->setChecked( use_3pt_briges );

    ui->plainTextEdit1_18->setPlainText(ttext[0]);
    ui->plainTextEdit253_18->setPlainText(tcut[0]);
    ui->plainTextEdit11_18->setPlainText(t1[0]);
    ui->plainTextEdit12_18->setPlainText(t2[0]);
    ui->plainTextEdit13_18->setPlainText(t3[0]);
    ui->plainTextEdit14_18->setPlainText(t4[0]);
    ui->spinBoxF1_18->setValue(ttextF[0].toLong());
    ui->spinBoxF253_18->setValue(tcutF[0].toLong());
    ui->spinBoxF11_18->setValue(t1F[0].toLong());
    ui->spinBoxF12_18->setValue(t2F[0].toLong());
    ui->spinBoxF13_18->setValue(t3F[0].toLong());
    ui->spinBoxF14_18->setValue(t4F[0].toLong());

    ui->plainTextEdit1_15->setPlainText(ttext[1]);
    ui->plainTextEdit253_15->setPlainText(tcut[1]);
    ui->plainTextEdit11_15->setPlainText(t1[1]);
    ui->plainTextEdit12_15->setPlainText(t2[1]);
    ui->plainTextEdit13_15->setPlainText(t3[1]);
    ui->plainTextEdit14_15->setPlainText(t4[1]);
    ui->spinBoxF1_15->setValue(ttextF[1].toLong());
    ui->spinBoxF253_15->setValue(tcutF[1].toLong());
    ui->spinBoxF11_15->setValue(t1F[1].toLong());
    ui->spinBoxF12_15->setValue(t2F[1].toLong());
    ui->spinBoxF13_15->setValue(t3F[1].toLong());
    ui->spinBoxF14_15->setValue(t4F[1].toLong());

    ui->plainTextEdit1_12->setPlainText(ttext[2]);
    ui->plainTextEdit253_12->setPlainText(tcut[2]);
    ui->plainTextEdit11_12->setPlainText(t1[2]);
    ui->plainTextEdit12_12->setPlainText(t2[2]);
    ui->plainTextEdit13_12->setPlainText(t3[2]);
    ui->plainTextEdit14_12->setPlainText(t4[2]);
    ui->spinBoxF1_12->setValue(ttextF[2].toLong());
    ui->spinBoxF253_12->setValue(tcutF[2].toLong());
    ui->spinBoxF11_12->setValue(t1F[2].toLong());
    ui->spinBoxF12_12->setValue(t2F[2].toLong());
    ui->spinBoxF13_12->setValue(t3F[2].toLong());
    ui->spinBoxF14_12->setValue(t4F[2].toLong());

    ui->plainTextEdit1_10->setPlainText(ttext[3]);
    ui->plainTextEdit253_10->setPlainText(tcut[3]);
    ui->plainTextEdit11_10->setPlainText(t1[3]);
    ui->plainTextEdit12_10->setPlainText(t2[3]);
    ui->plainTextEdit13_10->setPlainText(t3[3]);
    ui->plainTextEdit14_10->setPlainText(t4[3]);
    ui->spinBoxF1_10->setValue(ttextF[3].toLong());
    ui->spinBoxF253_10->setValue(tcutF[3].toLong());
    ui->spinBoxF11_10->setValue(t1F[3].toLong());
    ui->spinBoxF12_10->setValue(t2F[3].toLong());
    ui->spinBoxF13_10->setValue(t3F[3].toLong());
    ui->spinBoxF14_10->setValue(t4F[3].toLong());

}

void TppConfig::commitWin()
{
    QString prefix="";

    dir_output=ui->OutputFilePath->text();
    first_text = ui->textEditProlog->toPlainText();
    tool_engage = ui->textEditCutOn->toPlainText();
    tool_off = ui->textEditCutOff->toPlainText();
    prog_end = ui->textEditEpilog->toPlainText();
    MLS = ui->doubleSpinBox_PieLen->value();
    MLSinc = ui->spinBox_PieFIncrease->value();
    iniconf->setValue("CONF/DIR_OUTPUT", dir_output);
    iniconf->setValue("GGEN/START_TEXT", first_text);
    iniconf->setValue("GGEN/TOOL_ON", tool_engage);
    iniconf->setValue("GGEN/TOOL_OFF", tool_off);
    iniconf->setValue("GGEN/PROG_END", prog_end);
    iniconf->setValue("GGEN/MLS", MLS);
    iniconf->setValue("GGEN/MLSinc", MLSinc);

    t3bridges = ui->plainTextEdit13bridges_18->toPlainText();
    t3bridgesF = ui->spinBoxF13bridges_18->text();
    use_3pt_briges = ui->checkBox33ptBridges->isChecked();
    iniconf->setValue("GGEN/P3bridges", t3bridges);
    iniconf->setValue("GGEN/PF3bridges", t3bridgesF);
    iniconf->setValue("GGEN/Use3bridges", use_3pt_briges);

    int j=0;
    ttext[j] = ui->plainTextEdit1_18->toPlainText();
    tcut[j] = ui->plainTextEdit253_18->toPlainText();
    t1[j] = ui->plainTextEdit11_18->toPlainText();
    t2[j] = ui->plainTextEdit12_18->toPlainText();
    t3[j] = ui->plainTextEdit13_18->toPlainText();
    t4[j] = ui->plainTextEdit14_18->toPlainText();
    ttextF[j] = ui->spinBoxF1_18->text();
    tcutF[j] = ui->spinBoxF253_18->text();
    t1F[j] = ui->spinBoxF11_18->text();
    t2F[j] = ui->spinBoxF12_18->text();
    t3F[j] = ui->spinBoxF13_18->text();
    t4F[j] = ui->spinBoxF14_18->text();

    j=1;
    ttext[j] = ui->plainTextEdit1_15->toPlainText();
    tcut[j] = ui->plainTextEdit253_15->toPlainText();
    t1[j] = ui->plainTextEdit11_15->toPlainText();
    t2[j] = ui->plainTextEdit12_15->toPlainText();
    t3[j] = ui->plainTextEdit13_15->toPlainText();
    t4[j] = ui->plainTextEdit14_15->toPlainText();
    ttextF[j] = ui->spinBoxF1_15->text();
    tcutF[j] = ui->spinBoxF253_15->text();
    t1F[j] = ui->spinBoxF11_15->text();
    t2F[j] = ui->spinBoxF12_15->text();
    t3F[j] = ui->spinBoxF13_15->text();
    t4F[j] = ui->spinBoxF14_15->text();

    j=2;
    ttext[j] = ui->plainTextEdit1_12->toPlainText();
    tcut[j] = ui->plainTextEdit253_12->toPlainText();
    t1[j] = ui->plainTextEdit11_12->toPlainText();
    t2[j] = ui->plainTextEdit12_12->toPlainText();
    t3[j] = ui->plainTextEdit13_12->toPlainText();
    t4[j] = ui->plainTextEdit14_12->toPlainText();
    ttextF[j] = ui->spinBoxF1_12->text();
    tcutF[j] = ui->spinBoxF253_12->text();
    t1F[j] = ui->spinBoxF11_12->text();
    t2F[j] = ui->spinBoxF12_12->text();
    t3F[j] = ui->spinBoxF13_12->text();
    t4F[j] = ui->spinBoxF14_12->text();

    j=3;
    ttext[j] = ui->plainTextEdit1_10->toPlainText();
    tcut[j] = ui->plainTextEdit253_10->toPlainText();
    t1[j] = ui->plainTextEdit11_10->toPlainText();
    t2[j] = ui->plainTextEdit12_10->toPlainText();
    t3[j] = ui->plainTextEdit13_10->toPlainText();
    t4[j] = ui->plainTextEdit14_10->toPlainText();
    ttextF[j] = ui->spinBoxF1_10->text();
    tcutF[j] = ui->spinBoxF253_10->text();
    t1F[j] = ui->spinBoxF11_10->text();
    t2F[j] = ui->spinBoxF12_10->text();
    t3F[j] = ui->spinBoxF13_10->text();
    t4F[j] = ui->spinBoxF14_10->text();


    for (int i = 0; i < 4; ++i)
    {
        switch (i) {
            case 0: prefix="18"; break;
            case 1: prefix="15"; break;
            case 2: prefix="12"; break;
            case 3: prefix="10"; break;
            default:   prefix="def"; break;
        }
            iniconf->setValue("GGEN/Ptext"+prefix, ttext[i]);
            iniconf->setValue("GGEN/P1"+prefix, t1[i]);
            iniconf->setValue("GGEN/P2"+prefix, t2[i]);
            iniconf->setValue("GGEN/P3"+prefix, t3[i]);
            iniconf->setValue("GGEN/P4"+prefix, t4[i]);
            iniconf->setValue("GGEN/Pcut"+prefix, tcut[i]);

            iniconf->setValue("GGEN/PFtext"+prefix, ttextF[i]);
            iniconf->setValue("GGEN/PF1"+prefix, t1F[i]);
            iniconf->setValue("GGEN/PF2"+prefix, t2F[i]);
            iniconf->setValue("GGEN/PF3"+prefix, t3F[i]);
            iniconf->setValue("GGEN/PF4"+prefix, t4F[i]);
            iniconf->setValue("GGEN/PFcut"+prefix, tcutF[i]);
    }
}


void TppConfig::on_buttonBox_clicked(QAbstractButton *button)
{
    commitWin();
}

